@@ -1,6 +1,6 @@
 /*
  *  XMMP - LinuX MultiMedia Project ( www.frozenproductions.com )
- *  Copyright (c) 1999 - 2001 Arthur Kleer <kleer@frozenproductions.com>
+ *  Copyright (c) 1999 - 2002 Arthur Kleer <kleer@frozenproductions.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,11 +20,6 @@
 /*
  * lpfilterv.c
  * Manage Video Filter Chain ( and provide internal plugins for conversion )
- *
- * TODO:
- *	- xmm_FilterVChainStop(): segmentation fault at free( priv->image[0] ).
- *	  Very strange! I was not able to find a reason. I don't think it
- *	  is caused by a overflow of priv->image[0] itself.
  */
 
 #include <stdio.h>
@@ -97,14 +92,14 @@ XMM_PluginFilterVideo *xmm_FilterVideoOpen( void *_xmm, char *filename, XMM_Imag
 
   if( filename == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_INVALID_ARG, __FUNCTION__ "() filename == NULL\n" );
+	xmm_SetError( xmm, XMM_RET_INVALID_ARG, __FUNCTION__ "() filename == NULL\n" );
 	return NULL;
   }
 
   /* Register plugin */
   if(( tFV = (XMM_PluginFilterVideo *)xmm_PluginRegister( filename )) == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Unable to load plugin '%s'\n", filename );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Unable to load plugin '%s'\n", filename );
 	return NULL;
   }
 
@@ -165,27 +160,27 @@ int xmm_FilterVChainInit( void *_xmm )
   if( xmm == NULL )	return XMM_RET_ERROR;
 
   /* */
-  if( xmm->pVFpriv != NULL )
+  if( xmm->pFVCpriv != NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_ALLOC, __FUNCTION__ "() Video filter chain already initialized" );
+	xmm_SetError( xmm, XMM_RET_ALLOC, __FUNCTION__ "() Video filter chain already initialized" );
 	return XMM_RET_ERROR;
   }
 
   /* Allocate private data */
-  if(( xmm->pVFpriv = malloc( sizeof( struct priv_video_filter_s ))) == NULL )
+  if(( xmm->pFVCpriv = malloc( sizeof( struct priv_video_filter_s ))) == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_ALLOC, __FUNCTION__ "() Unable to allocate memory for video filter chain" );
+	xmm_SetError( xmm, XMM_RET_ALLOC, __FUNCTION__ "() Unable to allocate memory for video filter chain" );
 	return XMM_RET_ERROR;
   }
-  memset( xmm->pVFpriv, 0, sizeof( struct priv_video_filter_s ));
+  memset( xmm->pFVCpriv, 0, sizeof( struct priv_video_filter_s ));
 
-  priv = xmm->pVFpriv;
+  priv = xmm->pFVCpriv;
 
   /* Load internal plugins */
   priv->cVF_cfmt_default = (XMM_PluginFilterVideo *) xmm_PluginRegisterFromData((XMM_Plugin *) &plugin_info_cfmt );
   if( priv->cVF_cfmt_default == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Unable to register internal plugin" );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Unable to register internal plugin" );
 	return XMM_RET_ERROR;
   }
 
@@ -207,10 +202,10 @@ int xmm_FilterVChainExit( void *_xmm )
   if( xmm == NULL )	return XMM_RET_ERROR;
 
   /* */
-  priv = xmm->pVFpriv;
+  priv = xmm->pFVCpriv;
   if( priv == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain not initialized" );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain not initialized" );
 	return XMM_RET_ERROR;
   }
 
@@ -224,8 +219,8 @@ int xmm_FilterVChainExit( void *_xmm )
   xmmMutex_Destroy( priv->mutex );
 
   /* Free private chain data */
-  free( xmm->pVFpriv );
-  xmm->pVFpriv = NULL;
+  free( xmm->pFVCpriv );
+  xmm->pFVCpriv = NULL;
 
   return XMM_RET_OK;
 }
@@ -242,10 +237,10 @@ int xmm_FilterVChainLock( void *_xmm )
   if( xmm == NULL )	return XMM_RET_ERROR;
 
   /* */
-  priv = xmm->pVFpriv;
+  priv = xmm->pFVCpriv;
   if( priv == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain not initialized" );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain not initialized" );
 	return XMM_RET_ERROR;
   }
 
@@ -264,10 +259,10 @@ int xmm_FilterVChainUnlock( void *_xmm )
   if( xmm == NULL )	return XMM_RET_ERROR;
 
   /* */
-  priv = xmm->pVFpriv;
+  priv = xmm->pFVCpriv;
   if( priv == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain not initialized" );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain not initialized" );
 	return XMM_RET_ERROR;
   }
 
@@ -286,10 +281,10 @@ int xmm_FilterVChainActive( void *_xmm )
   if( xmm == NULL )	return XMM_RET_ERROR;
 
   /* */
-  priv = xmm->pVFpriv;
+  priv = xmm->pFVCpriv;
   if( priv == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain not initialized" );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain not initialized" );
 	return XMM_RET_ERROR;
   }
 
@@ -314,10 +309,10 @@ int xmm_FilterVChainInput( void *_xmm, uint32_t format, int width, int height, i
   if( xmm == NULL )	return XMM_RET_ERROR;
 
   /* */
-  priv = xmm->pVFpriv;
+  priv = xmm->pFVCpriv;
   if( priv == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain not initialized" );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain not initialized" );
 	return XMM_RET_ERROR;
   }
 
@@ -347,10 +342,10 @@ int xmm_FilterVChainOutput( void *_xmm, uint32_t format, int width, int height,
   if( xmm == NULL )	return XMM_RET_ERROR;
 
   /* */
-  priv = xmm->pVFpriv;
+  priv = xmm->pFVCpriv;
   if( priv == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain not initialized" );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain not initialized" );
 	return XMM_RET_ERROR;
   }
 
@@ -386,7 +381,7 @@ int xmm_FilterVChainOutput( void *_xmm, uint32_t format, int width, int height,
   /* Only conversion to RGB supported */
   if(( priv->dif.codec & XMM_GRAPH_FMT_RGB(0)) != XMM_GRAPH_FMT_RGB(0))
   {
-	xmm_SetError( xmm, XMM_ERR_NOTSUPPORTED, __FUNCTION__ "() Conversion only to RGB supported: %x [%s]\n", priv->dif.codec, xmm_FOURCC_string( priv->dif.codec ));
+	xmm_SetError( xmm, XMM_RET_NOTSUPPORTED, __FUNCTION__ "() Conversion only to RGB supported: %x [%s]\n", priv->dif.codec, xmm_FOURCC_string( priv->dif.codec ));
 
 	xmm_FilterVChainUnlock( xmm );
 	return XMM_RET_ERROR;
@@ -400,7 +395,7 @@ int xmm_FilterVChainOutput( void *_xmm, uint32_t format, int width, int height,
 
   if(( priv->image[0] = malloc( priv->image_size )) == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_ALLOC, __FUNCTION__ "() Unable to allocate memory for video filter chain" );
+	xmm_SetError( xmm, XMM_RET_ALLOC, __FUNCTION__ "() Unable to allocate memory for video filter chain" );
 	return XMM_RET_ERROR;
   }
 
@@ -437,7 +432,7 @@ int xmm_FilterVChainOutput( void *_xmm, uint32_t format, int width, int height,
 	if(( priv->pVF_cfmt = priv->cVF_cfmt_default->Open( xmm, &priv->sif, &priv->dif )) == NULL )
 	{
 	    strcpy( buffer, xmm_FOURCC_string( priv->dif.codec ));
-	    xmm_SetError( xmm, XMM_ERR_NOTSUPPORTED, __FUNCTION__ "() Conversion not supported: %x [%s] --> %x [%s]\n", priv->sif.codec, xmm_FOURCC_string( priv->sif.codec ), priv->dif.codec, buffer );
+	    xmm_SetError( xmm, XMM_RET_NOTSUPPORTED, __FUNCTION__ "() Conversion not supported: %x [%s] --> %x [%s]\n", priv->sif.codec, xmm_FOURCC_string( priv->sif.codec ), priv->dif.codec, buffer );
 
 	    xmm_FilterVChainUnlock( xmm );
 	    return XMM_RET_ERROR;
@@ -472,10 +467,10 @@ int xmm_FilterVChainData( void *_xmm, uint8_t *data[], int stride[], int x, int
   if( xmm == NULL )	return XMM_RET_ERROR;
 
   /* */
-  priv = xmm->pVFpriv;
+  priv = xmm->pFVCpriv;
   if( priv == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain not initialized" );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain not initialized" );
 	return XMM_RET_ERROR;
   }
 
@@ -518,10 +513,10 @@ int xmm_FilterVChain( void *_xmm, uint8_t *data[] )
   if( xmm == NULL )	return XMM_RET_ERROR;
 
   /* */
-  priv = xmm->pVFpriv;
+  priv = xmm->pFVCpriv;
   if( priv == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain not initialized" );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain not initialized" );
 	return XMM_RET_ERROR;
   }
 
@@ -545,10 +540,10 @@ int xmm_FilterVChainStop( void *_xmm )
   if( xmm == NULL )	return XMM_RET_ERROR;
 
   /* */
-  priv = xmm->pVFpriv;
+  priv = xmm->pFVCpriv;
   if( priv == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain not initialized" );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain not initialized" );
 	return XMM_RET_ERROR;
   }
 
@@ -600,22 +595,22 @@ int xmm_FilterVChainAdd( void *_xmm, XMM_PluginFilterVideo *pfv )
 
   if( pfv == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_INVALID_ARG, __FUNCTION__ "() filter plugin == NULL\n" );
+	xmm_SetError( xmm, XMM_RET_INVALID_ARG, __FUNCTION__ "() filter plugin == NULL\n" );
 	return XMM_RET_ERROR;
   }
 
   /* */
-  priv = xmm->pAFpriv;
+  priv = xmm->pFVCpriv;
   if( priv == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain not initialized." );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain not initialized." );
 	return XMM_RET_ERROR;
   }
 
   /* Try access to filter chain */
   if( xmmMutex_TryLock( priv->mutex ) != XMM_RET_OK )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain locked. Unable to add filter." );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain locked. Unable to add filter." );
 	return XMM_RET_ERROR;
   }
 
@@ -642,22 +637,22 @@ int xmm_FilterVChainAddList( void *_xmm, XMM_List *_pVideoFilterList )
 
   if( _pVideoFilterList == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_INVALID_ARG, __FUNCTION__ "() filter plugin list == NULL\n" );
+	xmm_SetError( xmm, XMM_RET_INVALID_ARG, __FUNCTION__ "() filter plugin list == NULL\n" );
 	return XMM_RET_ERROR;
   }
 
   /* */
-  priv = xmm->pAFpriv;
+  priv = xmm->pFVCpriv;
   if( priv == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain not initialized." );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain not initialized." );
 	return XMM_RET_ERROR;
   }
 
   /* Try access to filter chain */
   if( xmmMutex_TryLock( priv->mutex ) != XMM_RET_OK )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain locked. Unable to add filter." );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain locked. Unable to add filter." );
 	return XMM_RET_ERROR;
   }
 
@@ -682,17 +677,17 @@ int xmm_FilterVChainFree( void *_xmm )
   if( xmm == NULL )	return XMM_RET_ERROR;
 
   /* */
-  priv = xmm->pAFpriv;
+  priv = xmm->pFVCpriv;
   if( priv == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain not initialized." );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain not initialized." );
 	return XMM_RET_ERROR;
   }
 
   /* Try access to filter chain */
   if( xmmMutex_TryLock( priv->mutex ) != XMM_RET_OK )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, __FUNCTION__ "() Video Filter chain locked. Unable to add filter." );
+	xmm_SetError( xmm, XMM_RET_ERROR, __FUNCTION__ "() Video Filter chain locked. Unable to add filter." );
 	return XMM_RET_ERROR;
   }
 
@@ -837,29 +832,30 @@ static XMM_PluginFilterVideo *cfmt_Open( void *_xmm, XMM_ImageFormat *sif, XMM_I
     uint32_t		Dst;
     int (*Proc)( XMM_PluginFilterVideo *pfv, uint8_t *sdata[], int stride[], int width, int height, uint8_t *ddata[], int dx, int dy );
     int (*ProcS)( XMM_PluginFilterVideo *pfv, uint8_t *sdata[], int stride[], int width, int height, uint8_t *ddata[], int dx, int dy );
+    int			bpp;
   } aCv[] =
   {
-  { XMM_GRAPH_FMT_RGB(15), XMM_GRAPH_FMT_RGB(16), c_rgb15_rgb16, cS_rgb15_rgb16 },
-  { XMM_GRAPH_FMT_RGB(15), XMM_GRAPH_FMT_RGB(24), c_rgb15_rgb24, cS_rgb15_rgb24 },
-  { XMM_GRAPH_FMT_RGB(15), XMM_GRAPH_FMT_RGB(32), c_rgb15_rgb32, cS_rgb15_rgb32 },
-  { XMM_GRAPH_FMT_RGB(16), XMM_GRAPH_FMT_RGB(16), NULL,		 cS_rgb16_rgb16 },
-  { XMM_GRAPH_FMT_RGB(16), XMM_GRAPH_FMT_RGB(24), c_rgb16_rgb24, cS_rgb16_rgb24 },
-  { XMM_GRAPH_FMT_RGB(16), XMM_GRAPH_FMT_RGB(32), c_rgb16_rgb32, cS_rgb16_rgb32 },
-  { XMM_GRAPH_FMT_RGB(24), XMM_GRAPH_FMT_RGB(16), c_rgb24_rgb16, cS_rgb24_rgb16 },
-  { XMM_GRAPH_FMT_RGB(24), XMM_GRAPH_FMT_RGB(24), NULL,		 cS_rgb24_rgb24 },
-  { XMM_GRAPH_FMT_RGB(24), XMM_GRAPH_FMT_RGB(32), c_rgb24_rgb32, cS_rgb24_rgb32 },
-  { XMM_GRAPH_FMT_RGB(32), XMM_GRAPH_FMT_RGB(16), c_rgb32_rgb16, cS_rgb32_rgb16 },
-  { XMM_GRAPH_FMT_RGB(32), XMM_GRAPH_FMT_RGB(24), c_rgb32_rgb24, cS_rgb32_rgb24 },
-  { XMM_GRAPH_FMT_RGB(32), XMM_GRAPH_FMT_RGB(32), NULL,		 cS_rgb32_rgb32 },
-  { XMM_GRAPH_FMT_YV12, XMM_GRAPH_FMT_RGB(16), c_yv12_rgb16,	 cS_yv12_rgb16 },
-  { XMM_GRAPH_FMT_YV12, XMM_GRAPH_FMT_RGB(24), c_yv12_rgb24,	 cS_yv12_rgb24 },
-  { XMM_GRAPH_FMT_YV12, XMM_GRAPH_FMT_RGB(32), c_yv12_rgb32,	 cS_yv12_rgb32 },
+  { XMM_GRAPH_FMT_RGB(15), XMM_GRAPH_FMT_RGB(16), c_rgb15_rgb16, cS_rgb15_rgb16, 16 },
+  { XMM_GRAPH_FMT_RGB(15), XMM_GRAPH_FMT_RGB(24), c_rgb15_rgb24, cS_rgb15_rgb24, 24 },
+  { XMM_GRAPH_FMT_RGB(15), XMM_GRAPH_FMT_RGB(32), c_rgb15_rgb32, cS_rgb15_rgb32, 32 },
+  { XMM_GRAPH_FMT_RGB(16), XMM_GRAPH_FMT_RGB(16), NULL,		 cS_rgb16_rgb16, 16 },
+  { XMM_GRAPH_FMT_RGB(16), XMM_GRAPH_FMT_RGB(24), c_rgb16_rgb24, cS_rgb16_rgb24, 24 },
+  { XMM_GRAPH_FMT_RGB(16), XMM_GRAPH_FMT_RGB(32), c_rgb16_rgb32, cS_rgb16_rgb32, 32 },
+  { XMM_GRAPH_FMT_RGB(24), XMM_GRAPH_FMT_RGB(16), c_rgb24_rgb16, cS_rgb24_rgb16, 16 },
+  { XMM_GRAPH_FMT_RGB(24), XMM_GRAPH_FMT_RGB(24), NULL,		 cS_rgb24_rgb24, 24 },
+  { XMM_GRAPH_FMT_RGB(24), XMM_GRAPH_FMT_RGB(32), c_rgb24_rgb32, cS_rgb24_rgb32, 32 },
+  { XMM_GRAPH_FMT_RGB(32), XMM_GRAPH_FMT_RGB(16), c_rgb32_rgb16, cS_rgb32_rgb16, 16 },
+  { XMM_GRAPH_FMT_RGB(32), XMM_GRAPH_FMT_RGB(24), c_rgb32_rgb24, cS_rgb32_rgb24, 24 },
+  { XMM_GRAPH_FMT_RGB(32), XMM_GRAPH_FMT_RGB(32), NULL,		 cS_rgb32_rgb32, 32 },
+  { XMM_GRAPH_FMT_YV12, XMM_GRAPH_FMT_RGB(16), c_yv12_rgb16,	 cS_yv12_rgb16, 16 },
+  { XMM_GRAPH_FMT_YV12, XMM_GRAPH_FMT_RGB(24), c_yv12_rgb24,	 cS_yv12_rgb24, 24 },
+  { XMM_GRAPH_FMT_YV12, XMM_GRAPH_FMT_RGB(32), c_yv12_rgb32,	 cS_yv12_rgb32, 32 },
   };
 
   /* Allocate plugin data */
   if(( pVF = xmm_memdup_x( &plugin_info_cfmt, sizeof( XMM_PluginFilterVideo ), sizeof( struct priv_t ))) == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_ALLOC, __FUNCTION__ "() Unable to duplicate plugin_info" );
+	xmm_SetError( xmm, XMM_RET_ALLOC, __FUNCTION__ "() Unable to duplicate plugin_info" );
 	return NULL;
   }
 
@@ -886,6 +882,7 @@ static XMM_PluginFilterVideo *cfmt_Open( void *_xmm, XMM_ImageFormat *sif, XMM_I
 	{
 	    priv->Proc = pVF->Proc = aCv[i].Proc;
 	    priv->ProcS = aCv[i].ProcS;
+	    InitTables( aCv[i].bpp );
 	    return pVF;
 	}
   }
@@ -893,7 +890,7 @@ static XMM_PluginFilterVideo *cfmt_Open( void *_xmm, XMM_ImageFormat *sif, XMM_I
   free( pVF );
 
   strcpy( buffer, xmm_FOURCC_string( sif->codec ));
-  xmm_SetError( pVF->sys.xmm, XMM_ERR_NOTSUPPORTED, __FUNCTION__ "() Needed conversion not supported. %x [%s] --> %x [%s]", sif->codec, buffer, dif->codec, xmm_FOURCC_string( dif->codec ));
+  xmm_SetError( pVF->sys.xmm, XMM_RET_NOTSUPPORTED, __FUNCTION__ "() Needed conversion not supported. %x [%s] --> %x [%s]", sif->codec, buffer, dif->codec, xmm_FOURCC_string( dif->codec ));
 
   return (XMM_PluginFilterVideo *) NULL;
 }
@@ -931,8 +928,10 @@ static int cfmt_Control( XMM_PluginFilterVideo *filter, uint32_t cmd, uint32_t p
 	    break;
   }
 
-  if( cmd & XMM_CTLMASK_VFILTER )	return XMM_CTLRET_UNKNOWN;
-  return XMM_CTLRET_INVALID;	/* No VFILTER command */
+  if( cmd & XMM_CTLMASK_VFILTER )
+	return xmm_SetError( filter->sys.xmm, XMM_RET_NOTSUPPORTED, "(FVC-STD) cmd = 0x%x" );
+
+  return xmm_SetError( filter->sys.xmm, XMM_RET_INVALID_ARG, "(FVC-STD) cmd ( 0x%x )" );
 }
 
 /*
@@ -967,7 +966,6 @@ static long *pR_tab, *pG_tab, *pB_tab;
     CONVERSION	Conversion
 */
 
-
 #define CONV_FUNC( NAME, SRCTYPE, SRCMUL, SRCDIV, DSTTYPE, DSTMUL, VARS, CONVERSION ) \
 static int c_##NAME( XMM_PluginFilterVideo *conv, uint8_t *sdata[], int stride[], int width, int height, uint8_t *ddata[], int dx, int dy ) \
 { \
