@@ -1,6 +1,6 @@
 /*
  *  XMMP - LinuX MultiMedia Project ( www.frozenproductions.com )
- *  Copyright (c) 1999 - 2001 Arthur Kleer <kleer@frozenproductions.com>
+ *  Copyright (c) 1999 - 2002 Arthur Kleer <kleer@frozenproductions.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -122,7 +122,7 @@ static XMM_PluginGraph *sdl_Init( void *xmm )
    */
   if(( graph = xmm_memdup_x( &plugin_info, sizeof( XMM_PluginGraph ), sizeof( struct priv_t ))) == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_ALLOC, "(SDL) Unable to duplicate plugin_info" );
+	xmm_SetError( xmm, XMM_RET_ALLOC, "(SDL) Unable to duplicate plugin_info" );
 	return NULL;
   }
 
@@ -136,7 +136,7 @@ static XMM_PluginGraph *sdl_Init( void *xmm )
   /* Initialize SDL */
   if( SDL_Init( SDL_INIT_VIDEO ) < 0 )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, "(SDL) Video init error: %s.", SDL_GetError());
+	xmm_SetError( xmm, XMM_RET_ERROR, "(SDL) Video init error: %s.", SDL_GetError());
 	return NULL;
   }
 
@@ -215,99 +215,111 @@ static int sdl_Control( XMM_PluginGraph *graph, uint32_t cmd, uint32_t param, vo
 
   switch( cmd )
   {
-    case XMM_CTLQUERY_HWSCALE:
-	    return XMM_CTLRET_FALSE;
+	case XMM_CTLQUERY_HWSCALE:
+		return XMM_CTLRET_FALSE;
 	    
-    case XMM_CTLQUERY_YFLIP:
-	    return XMM_CTLRET_FALSE;
+	case XMM_CTLQUERY_YFLIP:
+		return XMM_CTLRET_FALSE;
 
-    case XMM_CTLQUERY_FULLSCREEN:
-	    return XMM_CTLRET_TRUE;
+	case XMM_CTLQUERY_FULLSCREEN:
+		return XMM_CTLRET_TRUE;
 
-    case XMM_CTLQUERY_GFORMAT:
-	    if( param == XMM_GRAPH_FMT_RGB( priv->pixelsize * 8 ))	return XMM_CTLRET_TRUE;
-	    if( param == XMM_GRAPH_FMT_YV12 )	return XMM_CTLRET_TRUE;
-	    if( param == XMM_GRAPH_FMT_IYUV )	return XMM_CTLRET_TRUE;
-	    if( param == XMM_GRAPH_FMT_YUY2 )	return XMM_CTLRET_TRUE;
-	    if( param == XMM_GRAPH_FMT_UYVY )	return XMM_CTLRET_TRUE;
-	    if( param == XMM_GRAPH_FMT_YVYU )	return XMM_CTLRET_TRUE;
-	    return XMM_CTLRET_FALSE;
+	case XMM_CTLQUERY_GFORMAT:
+		if((uint32_t)data == XMM_GRAPH_FMT_RGB( priv->pixelsize * 8 ))	return XMM_CTLRET_TRUE;
+		if((uint32_t)data == XMM_GRAPH_FMT_YV12 )	return XMM_CTLRET_TRUE;
+		if((uint32_t)data == XMM_GRAPH_FMT_IYUV )	return XMM_CTLRET_TRUE;
+		if((uint32_t)data == XMM_GRAPH_FMT_YUY2 )	return XMM_CTLRET_TRUE;
+		if((uint32_t)data == XMM_GRAPH_FMT_UYVY )	return XMM_CTLRET_TRUE;
+		if((uint32_t)data == XMM_GRAPH_FMT_YVYU )	return XMM_CTLRET_TRUE;
+		return XMM_CTLRET_FALSE;
 
-    case XMM_CTLQUERY_SURFACE_READ:
-	    return XMM_CTLRET_TRUE;
-    
-    case XMM_CTLGET_GFORMAT:
-	    if(( priv->sformat & XMM_GRAPH_FMT_RGB(0)) == XMM_GRAPH_FMT_RGB(0))
-		*((uint32_t *)data) = xmmFOURCC( 'R','G','B', priv->pixelsize * 8 );
-	    else
-		*((uint32_t *)data) = xmmFOURCC( 'Y','U','Y','2' );
+	case XMM_CTLQUERY_SURFACE_READ:
+		return XMM_CTLRET_TRUE;
 
-	    return XMM_CTLRET_ARG;		/* Result in arg */
-	    
-    case XMM_CTLGET_SURFACE:
-	    surface = (XMM_ControlSurface *)data;
+	case XMM_CTLQUERY_CONFIG:
+		return XMM_CTLRET_FALSE;
 
-	    /* Surface only for internal use, if filter chain active */
-	    if( xmm_FilterVChainActive( graph->sys.xmm ) == 1 )	return XMM_CTLRET_FALSE;
+	case XMM_CTLGET_GFORMAT:
+		if(( priv->sformat & XMM_GRAPH_FMT_RGB(0)) == XMM_GRAPH_FMT_RGB(0))
+		    *((uint32_t *)data) = xmmFOURCC( 'R','G','B', priv->pixelsize * 8 );
+		else
+		    *((uint32_t *)data) = xmmFOURCC( 'Y','U','Y','2' );
 
-	    /* Return surface type */
-	    surface->fourcc = priv->dformat;
+		return XMM_CTLRET_ARG;		/* Result in arg */
 
-	    /* Return surface pointer and stride */
-    	    if( priv->dyuv == 1 )
-	    {
-		surface->data[0] = priv->overlay->pixels[0];
-		surface->data[1] = NULL;
-		surface->data[2] = NULL;
-		surface->stride[0] = priv->overlay->w * 2;
-	    }
-	    else if( priv->dyuv == 2 )
-	    {
-		surface->data[0] = priv->overlay->pixels[0];
-		surface->data[1] = priv->overlay->pixels[1];
-		surface->data[2] = priv->overlay->pixels[2];
-		surface->stride[0] = priv->overlay->w;
-		surface->stride[1] = priv->overlay->w >> 2;
-		surface->stride[2] = priv->overlay->w >> 2;
-	    }
-	    else
-	    {
-		surface->data[0] = priv->surface->pixels;
-		surface->stride[0] = priv->surface->w * priv->pixelsize;
-	    }
+	case XMM_CTLGET_SURFACE:
+		surface = (XMM_ControlSurface *)data;
 
-	    return XMM_CTLRET_ARG;		/* Result in arg */
+		/* Surface only for internal use, if filter chain active */
+		if( xmm_FilterVChainActive( graph->sys.xmm ) == 1 )	return XMM_CTLRET_FALSE;
 
-    case XMM_CTLGET_SCREEN:
-	    return XMM_CTLRET_NOTSUPPORTED;
+		/* Return surface type */
+		surface->fourcc = priv->dformat;
 
-    case XMM_CTLSET_SCALE:
-	    scale = (XMM_ControlScale *)data;
+		/* Return surface pointer and stride */
+    		if( priv->dyuv == 1 )
+		{
+		    surface->data[0] = priv->overlay->pixels[0];
+		    surface->data[1] = NULL;
+		    surface->data[2] = NULL;
+		    surface->stride[0] = priv->overlay->w * 2;
+		}
+		else if( priv->dyuv == 2 )
+		{
+		    surface->data[0] = priv->overlay->pixels[0];
+		    surface->data[1] = priv->overlay->pixels[1];
+		    surface->data[2] = priv->overlay->pixels[2];
+		    surface->stride[0] = priv->overlay->w;
+		    surface->stride[1] = priv->overlay->w >> 2;
+		    surface->stride[2] = priv->overlay->w >> 2;
+		}
+		else
+		{
+		    surface->data[0] = priv->surface->pixels;
+		    surface->stride[0] = priv->surface->w * priv->pixelsize;
+		}
+
+		return XMM_CTLRET_ARG;		/* Result in arg */
+
+	case XMM_CTLGET_SCREEN:
+		return XMM_RET_NOTSUPPORTED;
+
+	case XMM_CTLSET_SCALE:
+		scale = (XMM_ControlScale *)data;
 #if 0
-	    if(( scale->width - priv->dwidth ) || ( scale->height - priv->dheight ))
+		if(( scale->width - priv->dwidth ) || ( scale->height - priv->dheight ))
 #endif
-		if( sdl_Resize( graph, priv, scale->width, scale->height, scale->flags ) >= 0 )
-		    return XMM_CTLRET_TRUE;
+		    if( sdl_Resize( graph, priv, scale->width, scale->height, scale->flags ) >= 0 )
+			return XMM_CTLRET_TRUE;
 
-	    return XMM_CTLRET_ERROR;
+		return XMM_CTLRET_FALSE;
 
-    case XMM_CTLSET_POSITION:
-	    return XMM_CTLRET_FALSE;		/* TODO: set window position */
+	case XMM_CTLSET_POSITION:
+		return XMM_CTLRET_FALSE;		/* TODO: set window position */
 
-    case XMM_CTLSET_TITLE:
-	    if( data != NULL )	SDL_WM_SetCaption((char *)data, (char *)data );
-	    else	SDL_WM_SetCaption( XMM_PROJECT_NAME, XMM_PROJECT_NAME );
-	    return XMM_CTLRET_TRUE;
+	case XMM_CTLSET_TITLE:
+		if( data != NULL )	SDL_WM_SetCaption((char *)data, (char *)data );
+		else	SDL_WM_SetCaption( XMM_PROJECT_NAME, XMM_PROJECT_NAME );
+		return XMM_CTLRET_TRUE;
 
-    case XMM_CTLSET_FRAMERATE:
-	    return XMM_CTLRET_NOTSUPPORTED;
+	case XMM_CTLSET_FRAMERATE:
+		return XMM_RET_NOTSUPPORTED;
 
-    default:
-	    break;
+	/* Dialogues */
+	case XMM_CTLDLG_QUERY:
+		return XMM_CTLRET_FALSE;
+
+	case XMM_CTLDLG_DISPLAY:
+		return XMM_RET_NOTSUPPORTED;
+
+	default:
+		break;
   }
 
-  if( cmd & XMM_CTLMASK_GRAPH )	return XMM_CTLRET_UNKNOWN;
-  return XMM_CTLRET_INVALID;	/* No GRAPH command */
+  if( cmd & XMM_CTLMASK_GRAPH )
+	return xmm_SetError( graph->sys.xmm, XMM_RET_NOTSUPPORTED, "(SDL) cmd = 0x%x" );
+
+  return xmm_SetError( graph->sys.xmm, XMM_RET_INVALID_ARG, "(SDL) cmd ( 0x%x )" );
 }
 
 /*
@@ -335,7 +347,7 @@ static int sdl_Start( XMM_PluginGraph *graph, int width, int height, uint32_t fo
   /*
    * Check whether conversion needed
    */
-  if( sdl_Control( graph, XMM_CTLQUERY_GFORMAT, priv->sformat, NULL ) != XMM_CTLRET_TRUE )
+  if( sdl_Control( graph, XMM_CTLQUERY_GFORMAT, 0, (void *)priv->sformat ) != XMM_CTLRET_TRUE )
   {
 	sdl_Control( graph, XMM_CTLGET_GFORMAT, 0, &priv->dformat );
   }
@@ -538,7 +550,7 @@ XMM_PluginGraph	plugin_info = {	{ NULL,
 				"",
 				"SDL",
 				"Graph: SDL",
-				"Copyright (c) 2000, 2001 by Arthur Kleer",
+				"Copyright (c) 1999 Arthur Kleer",
 				NULL, NULL },
 				sdl_Init, sdl_Close, sdl_Control, sdl_Start,
 				sdl_Draw, sdl_Blit };
@@ -565,7 +577,7 @@ static int sdl_Resize( XMM_PluginGraph *graph, struct priv_t *priv, int width, i
   if(( !width || !height ) && !flags )
   {
 	xmmMutex_Unlock( priv->mutex );
-	return xmm_SetError( graph->sys.xmm, XMM_ERR_UNKNOWN, "(SDL) Wrong parameters: width = 0, height = 0 only with flags ( FULLSCREEN )" );
+	return xmm_SetError( graph->sys.xmm, XMM_RET_ERROR, "(SDL) Wrong parameters: width = 0, height = 0 only with flags ( FULLSCREEN )" );
   }
 
   if(( flags & XMM_GRAPH_RESIZE_ORIG ) == XMM_GRAPH_RESIZE_ORIG )
@@ -612,7 +624,7 @@ static int sdl_Resize( XMM_PluginGraph *graph, struct priv_t *priv, int width, i
 	if( vm == -1 )
 	{
 	    xmmMutex_Unlock( priv->mutex );
-	    return xmm_SetError( graph->sys.xmm, XMM_ERR_UNKNOWN, "(SDL) Video init error: No matching mode found." );
+	    return xmm_SetError( graph->sys.xmm, XMM_RET_ERROR, "(SDL) Video init error: No matching mode found." );
 	}
 
 	if( priv->surface )	SDL_FreeSurface( priv->surface );
@@ -620,7 +632,7 @@ static int sdl_Resize( XMM_PluginGraph *graph, struct priv_t *priv, int width, i
 	if(( priv->surface = SDL_SetVideoMode( priv->sdl_fmodes[vm]->w, priv->sdl_fmodes[vm]->h, priv->pixelsize * 8, priv->sdl_fflags )) == NULL )
 	{
 	    xmmMutex_Unlock( priv->mutex );
-	    return xmm_SetError( graph->sys.xmm, XMM_ERR_UNKNOWN, "(SDL) Coludn't set video mode to %xx%x: %s", priv->sdl_fmodes[vm]->w, priv->sdl_fmodes[vm]->h, SDL_GetError());
+	    return xmm_SetError( graph->sys.xmm, XMM_RET_ERROR, "(SDL) Coludn't set video mode to %xx%x: %s", priv->sdl_fmodes[vm]->w, priv->sdl_fmodes[vm]->h, SDL_GetError());
 	}
 
 	/* Stretch to full screen ? */
@@ -639,7 +651,7 @@ static int sdl_Resize( XMM_PluginGraph *graph, struct priv_t *priv, int width, i
 	if(( priv->surface = SDL_SetVideoMode( width, height, priv->pixelsize * 8, priv->sdl_flags )) == NULL )
 	{
 	    xmmMutex_Unlock( priv->mutex );
-	    return xmm_SetError( graph->sys.xmm, XMM_ERR_UNKNOWN, "(SDL) Coludn't set video mode to %xx%x: %s", width, height, SDL_GetError());
+	    return xmm_SetError( graph->sys.xmm, XMM_RET_ERROR, "(SDL) Coludn't set video mode to %xx%x: %s", width, height, SDL_GetError());
 	}
 
 	/* Set window title */
