@@ -1,6 +1,6 @@
 /*
  *  XMMP - LinuX MultiMedia Project ( www.frozenproductions.com )
- *  Copyright (c) 1999 - 2001 Arthur Kleer <kleer@frozenproductions.com>
+ *  Copyright (c) 1999 - 2002 Arthur Kleer <kleer@frozenproductions.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/*
- * x11.c
+/* x11.c
  * X11 graph output
  */
 
@@ -81,6 +80,7 @@ extern XMM_PluginGraph	plugin_info;
 
 static int x11_Resize( XMM_PluginGraph *graph, int width, int height, int flags );
 static int x11_CheckForEvent( XMM_PluginGraph *graph );
+static int x11_screen_saver( Display *display, int timeout );
 
 /* XF86-DGA Extension 1.x */
 #ifdef HAVE_XF86DGA1x
@@ -107,6 +107,12 @@ void xv_exit( Display *display, xv_data_t *xvd );
 int xv_start( Display *display, xv_data_t *xvd, uint32_t format, int xsize, int ysize );
 #endif
 
+/* Xdpms extension */
+#ifdef HAVE_XDPMS
+int x11_dpms_off( Display *display );
+int x11_dpms_on( Display *display );
+#endif
+
 /*
  * Types
  */
@@ -136,6 +142,8 @@ struct priv_t
     int			dwidth;
     int			dheight;
     uint32_t		dformat;
+    int			dxoff;
+    int			dyoff;
 
     /* Screen size */
     int			wwidth;
@@ -172,6 +180,10 @@ struct priv_t
     uint32_t		compOffset[3];
 #endif
 
+#ifdef HAVE_XDPMS
+    int			dpms_disabled;
+#endif
+    int			ss_timeout;
 };
 
 /*
@@ -190,7 +202,7 @@ static XMM_PluginGraph *x11_Init( void *xmm )
    */
   if(( graph = xmm_memdup_x( &plugin_info, sizeof( XMM_PluginGraph ), sizeof( struct priv_t ))) == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_ALLOC, "(X11) Unable to duplicate plugin_info" );
+	xmm_SetError( xmm, XMM_RET_ALLOC, "(X11) Unable to duplicate plugin_info" );
 	return NULL;
   }
 
@@ -206,7 +218,7 @@ static XMM_PluginGraph *x11_Init( void *xmm )
    */
   if(( priv->display = XOpenDisplay( NULL )) == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, "(X11) Couldn't open display." );
+	xmm_SetError( xmm, XMM_RET_ERROR, "(X11) Couldn't open display." );
 	return NULL;
   }
 
@@ -283,6 +295,12 @@ static void x11_Close( XMM_PluginGraph *graph )
   }
 #endif
 
+#ifdef HAVE_XDPMS
+  if( priv->dpms_disabled )	x11_dpms_on( priv->display );
+#endif
+
+  if( priv->ss_timeout )	x11_screen_saver( priv->display, priv->ss_timeout );
+
   /*
    * Free X stuff
    */
@@ -323,138 +341,150 @@ static int x11_Control( XMM_PluginGraph *graph, uint32_t cmd, uint32_t param, vo
 
   switch( cmd )
   {
-    case XMM_CTLQUERY_HWSCALE:
-	    return XMM_CTLRET_FALSE;
+	case XMM_CTLQUERY_HWSCALE:
+		return XMM_CTLRET_FALSE;
 	    
-    case XMM_CTLQUERY_YFLIP:
-	    return XMM_CTLRET_FALSE;
+	case XMM_CTLQUERY_YFLIP:
+		return XMM_CTLRET_FALSE;
 
-    case XMM_CTLQUERY_FULLSCREEN:
-	    return XMM_CTLRET_TRUE;
+	case XMM_CTLQUERY_FULLSCREEN:
+		return XMM_CTLRET_TRUE;
 
-    case XMM_CTLQUERY_GFORMAT:
+	case XMM_CTLQUERY_GFORMAT:
 #ifdef HAVE_XV
-	    /* YCrCb formats */
-	    if( priv->xvd.aFormat != NULL )
-	    {
-		for( i = 0; i < priv->xvd.formats; i++ )
-		    if( param == priv->xvd.aFormat[i] )	break;
+		/* YCrCb formats */
+		if( priv->xvd.aFormat != NULL )
+		{
+		    for( i = 0; i < priv->xvd.formats; i++ )
+			if((uint32_t) data == priv->xvd.aFormat[i] )	break;
 
-		if( i != priv->xvd.formats )	return XMM_CTLRET_TRUE;
-	    }
+		    if( i != priv->xvd.formats )	return XMM_CTLRET_TRUE;
+		}
 #endif
-	    /* RGB formats */
-	    if( priv->bgr )
-	    {
-		if( param == xmmFOURCC( 'B','G','R', priv->pixelsize * 8 ))
+		/* RGB formats */
+		if( priv->bgr )
+		{
+		    if((uint32_t) data == xmmFOURCC( 'B','G','R', priv->pixelsize * 8 ))
 			return XMM_CTLRET_TRUE;
-	    }
-	    else
-	    {
-		if( param == xmmFOURCC( 'R','G','B', priv->pixelsize * 8 ))
+		}
+		else
+		{
+		    if((uint32_t) data == xmmFOURCC( 'R','G','B', priv->pixelsize * 8 ))
 			return XMM_CTLRET_TRUE;
-	    }
+		}
 
-	    return XMM_CTLRET_FALSE;
+		return XMM_CTLRET_FALSE;
 
-    case XMM_CTLQUERY_SURFACE_READ:
-	    return XMM_CTLRET_TRUE;
+	case XMM_CTLQUERY_SURFACE_READ:
+		return XMM_CTLRET_TRUE;
+
+	case XMM_CTLQUERY_CONFIG:
+		return XMM_CTLRET_FALSE;
+
+	/* Dialogues */
+	case XMM_CTLDLG_QUERY:
+		return XMM_CTLRET_FALSE;
+
+	case XMM_CTLDLG_DISPLAY:
+		return XMM_RET_NOTSUPPORTED;
     
-    case XMM_CTLGET_GFORMAT:
+	case XMM_CTLGET_GFORMAT:
 #ifdef HAVE_XV
-	    /* YCrCb formats */
-	    if( priv->xvd.aFormat != NULL )
-	    {
-		*((uint32_t *)data) = priv->xvd.aFormat[0];
-	    }
-	    else
+		/* YCrCb formats */
+		if( priv->xvd.aFormat != NULL )
+		{
+		    *((uint32_t *)data) = priv->xvd.aFormat[0];
+		}
+		else
 #endif
-	    if( priv->bgr )
-	    {
-		*((uint32_t *)data) = xmmFOURCC( 'B','G','R', priv->pixelsize * 8 );
-	    }
-	    else
-	    {
-		*((uint32_t *)data) = xmmFOURCC( 'R','G','B', priv->pixelsize * 8 );
-	    }
+		if( priv->bgr )
+		{
+		    *((uint32_t *)data) = xmmFOURCC( 'B','G','R', priv->pixelsize * 8 );
+		}
+		else
+		{
+		    *((uint32_t *)data) = xmmFOURCC( 'R','G','B', priv->pixelsize * 8 );
+		}
 
-	    return XMM_CTLRET_ARG;		/* Result in arg */
+		return XMM_CTLRET_ARG;		/* Result in arg */
 
-    case XMM_CTLGET_SURFACE:
-	    surface = (XMM_ControlSurface *)data;
+	case XMM_CTLGET_SURFACE:
+		surface = (XMM_ControlSurface *)data;
 
-	    /* Surface only for internal use, if filter chain active */
-	    if( xmm_FilterVChainActive( graph->sys.xmm ) == 1 )	return XMM_CTLRET_FALSE;
+		/* Surface only for internal use, if filter chain active */
+		if( xmm_FilterVChainActive( graph->sys.xmm ) == 1 )	return XMM_CTLRET_FALSE;
 
-	    /* Return surface type */
-	    surface->fourcc = priv->dformat;
+		/* Return surface type */
+		surface->fourcc = priv->dformat;
 
-	    /* Return surface pointer */
+		/* Return surface pointer */
 #ifdef HAVE_XV
-    	    if( priv->dyuv == 1 )
-	    {
-		surface->data[0] = priv->xvd.image->data;
-		surface->data[1] = NULL;
-		surface->data[2] = NULL;
-		surface->stride[0] = priv->xvd.image->width * 2;
-	    }
-	    else if( priv->dyuv == 2 )
-	    {
-		surface->data[0] = priv->xvd.image->data;
-		surface->data[1] = (uint8_t *)priv->xvd.image->data + priv->ysize;
-		surface->data[2] = (uint8_t *)priv->xvd.image->data + priv->ysize + priv->uvsize;
-		surface->stride[0] = priv->xvd.image->width;
-		surface->stride[1] = priv->xvd.image->width >> 2;
-		surface->stride[2] = priv->xvd.image->width >> 2;
-	    }
-	    else
-	    {
+    		if( priv->dyuv == 1 )
+		{
+		    surface->data[0] = priv->xvd.image->data;
+		    surface->data[1] = NULL;
+		    surface->data[2] = NULL;
+		    surface->stride[0] = priv->xvd.image->width * 2;
+		}
+		else if( priv->dyuv == 2 )
+		{
+		    surface->data[0] = priv->xvd.image->data;
+		    surface->data[1] = (uint8_t *)priv->xvd.image->data + priv->ysize;
+		    surface->data[2] = (uint8_t *)priv->xvd.image->data + priv->ysize + priv->uvsize;
+		    surface->stride[0] = priv->xvd.image->width;
+		    surface->stride[1] = priv->xvd.image->width >> 2;
+		    surface->stride[2] = priv->xvd.image->width >> 2;
+		}
+		else
+		{
 #endif
-		surface->data[0] = priv->image_data;
-		surface->stride[0] = priv->wwidth * priv->pixelsize;
+		    surface->data[0] = priv->image_data;
+		    surface->stride[0] = priv->wwidth * priv->pixelsize;
 #ifdef HAVE_XV
-	    }
+		}
 #endif
 
-	    return XMM_CTLRET_ARG;		/* Result in arg */
+		return XMM_CTLRET_ARG;		/* Result in arg */
 
     case XMM_CTLGET_SCREEN:
-	    rect = (XMM_ControlRect *) data;
-	    rect->width = DisplayWidth( priv->display, priv->screen );
-	    rect->height = DisplayHeight( priv->display, priv->screen );
+		rect = (XMM_ControlRect *) data;
+		rect->width = DisplayWidth( priv->display, priv->screen );
+		rect->height = DisplayHeight( priv->display, priv->screen );
 
-	    return XMM_CTLRET_ARG;		/* Result in arg */
+		return XMM_CTLRET_ARG;		/* Result in arg */
 
     case XMM_CTLSET_SCALE:
-	    scale = (XMM_ControlScale *)data;
+		scale = (XMM_ControlScale *)data;
 #if 0
-	    if(( scale->width - priv->dwidth ) || ( scale->height - priv->dheight ))
+		if(( scale->width - priv->dwidth ) || ( scale->height - priv->dheight ))
 #endif
-		if( x11_Resize( graph, scale->width, scale->height, scale->flags ) >= 0 )
-		    return XMM_CTLRET_TRUE;
+		    if( x11_Resize( graph, scale->width, scale->height, scale->flags ) >= 0 )
+			return XMM_CTLRET_TRUE;
 
-	    return XMM_CTLRET_FALSE;
+		return XMM_CTLRET_FALSE;
 
-    case XMM_CTLSET_POSITION:
-	    XMoveWindow( priv->display, priv->window,  ((XMM_ControlRect *)data)->x, ((XMM_ControlRect *)data)->y );
-	    return XMM_CTLRET_TRUE;
+	case XMM_CTLSET_POSITION:
+		XMoveWindow( priv->display, priv->window,  ((XMM_ControlRect *)data)->x, ((XMM_ControlRect *)data)->y );
+		return XMM_CTLRET_TRUE;
 
-    case XMM_CTLSET_TITLE:
-	    if( data == NULL )
-		XSetStandardProperties( priv->display, priv->window, XMM_PROJECT_NAME, XMM_PROJECT_NAME, None, NULL, 0, NULL );
-	    else
-		XSetStandardProperties( priv->display, priv->window, (char *)data, (char *)data, None, NULL, 0, NULL );
-	    return XMM_CTLRET_TRUE;
+	case XMM_CTLSET_TITLE:
+		if( data == NULL )
+		    XSetStandardProperties( priv->display, priv->window, XMM_PROJECT_NAME, XMM_PROJECT_NAME, None, NULL, 0, NULL );
+		else
+		    XSetStandardProperties( priv->display, priv->window, (char *)data, (char *)data, None, NULL, 0, NULL );
+		return XMM_CTLRET_TRUE;
 
-    case XMM_CTLSET_FRAMERATE:
-	    return XMM_CTLRET_NOTSUPPORTED;
+	case XMM_CTLSET_FRAMERATE:
+		return XMM_RET_NOTSUPPORTED;
 
-    default:
-	    break;
+	default:
+		break;
   }
 
-  if( cmd & XMM_CTLMASK_GRAPH )	return XMM_CTLRET_UNKNOWN;
-  return XMM_CTLRET_INVALID;	/* No GRAPH command */
+  if( cmd & XMM_CTLMASK_GRAPH )
+	return xmm_SetError( graph->sys.xmm, XMM_RET_NOTSUPPORTED, "(X11) cmd = 0x%x" );
+
+  return xmm_SetError( graph->sys.xmm, XMM_RET_INVALID_ARG, "(X11) cmd ( 0x%x )" );
 }
 
 /*
@@ -469,6 +499,7 @@ static int x11_Start( XMM_PluginGraph *graph, int width, int height, uint32_t fo
   XColor	fg, bg;
   char		data[8]={ 0, 0, 0, 0, 0, 0, 0, 0 };
   char		buffer[5];
+  int		ret;
   uint32_t	aFmt[] = 
   {
     XMM_GRAPH_FMT_RGB( priv->pixelsize * 8 ),
@@ -479,7 +510,7 @@ static int x11_Start( XMM_PluginGraph *graph, int width, int height, uint32_t fo
   if(( DisplayWidth( priv->display, priv->screen ) < width ) ||
 	( DisplayHeight( priv->display, priv->screen ) < height ))
   {
-	return xmm_SetError( graph->sys.xmm, XMM_ERR_UNKNOWN, "(X11) Needs at least %ix%i to run.\n", width, height );
+	return xmm_SetError( graph->sys.xmm, XMM_RET_ERROR, "(X11) Needs at least %ix%i to run.\n", width, height );
   }
 
   /* Create Window */
@@ -505,9 +536,6 @@ static int x11_Start( XMM_PluginGraph *graph, int width, int height, uint32_t fo
   /* no interest in Exposure */
   XSelectInput( priv->display, priv->window, KeyPressMask | KeyReleaseMask );
 
-  /* no auto repeat for keys */
-  XAutoRepeatOff( priv->display );
-
   /* Disable mouse pointer ( set it to nothing ) */
   fg.pixel = WhitePixel( priv->display, priv->screen );
   XQueryColor( priv->display, DefaultColormap( priv->display, priv->screen ), &fg );
@@ -556,7 +584,7 @@ static int x11_Start( XMM_PluginGraph *graph, int width, int height, uint32_t fo
   /*
    * Check whether conversion needed
    */
-  if( x11_Control( graph, XMM_CTLQUERY_GFORMAT, priv->sformat, NULL ) != XMM_CTLRET_TRUE )
+  if( x11_Control( graph, XMM_CTLQUERY_GFORMAT, 0, (void *)priv->sformat ) != XMM_CTLRET_TRUE )
   {
 	x11_Control( graph, XMM_CTLGET_GFORMAT, 0, &priv->dformat );
 
@@ -598,10 +626,11 @@ static int x11_Start( XMM_PluginGraph *graph, int width, int height, uint32_t fo
 #endif
 
   strcpy( buffer, xmm_FOURCC_string( priv->dformat ));
+
 #ifdef HAVE_XV
-  xmm_logging( 2, "X11! Initialized. Format: %x [%s] -> %x [%s] Overlay: %s\n", priv->sformat, xmm_FOURCC_string( priv->sformat ), priv->dformat, buffer, priv->dyuv ? "YUV" : "no" );
+  xmm_logging( 2, "X11! Init. Format: %x [%s] -> %x [%s] YF = %s Overlay: %s\n", priv->sformat, xmm_FOURCC_string( priv->sformat ), priv->dformat, buffer, ( priv->gflags & XMM_GRAPH_FLAG_YFLIP ) ? "yes" : "no", priv->dyuv ? "YUV" : "no" );
 #else
-  xmm_logging( 2, "X11! Initialized. Format: %x [%s] -> %x [%s]\n", priv->sformat, xmm_FOURCC_string( priv->sformat ), priv->dformat, buffer );
+  xmm_logging( 2, "X11! Init. Format: %x [%s] -> %x [%s] YF = %s\n", priv->sformat, xmm_FOURCC_string( priv->sformat ), priv->dformat, buffer, ( priv->gflags & XMM_GRAPH_FLAG_YFLIP ) ? "yes" : "no" );
 #endif
 
   /* Check periodicaly for events */
@@ -609,7 +638,20 @@ static int x11_Start( XMM_PluginGraph *graph, int width, int height, uint32_t fo
   xmmTimer_Add((XMM_TimerProc) x11_CheckForEvent, (void *)graph );
 #endif
 
-  return x11_Resize( graph, width, height, 0 );
+
+  ret = x11_Resize( graph, width, height, 0 );
+
+#ifdef HAVE_XDPMS
+  if( ret == XMM_RET_OK )	priv->dpms_disabled = x11_dpms_off( priv->display );
+#endif
+
+  /* Disable screen-saver */
+  priv->ss_timeout = x11_screen_saver( priv->display, 0 );
+
+  /* no auto repeat for keys */
+  XAutoRepeatOff( priv->display );
+
+  return ret;  
 }
 
 /*
@@ -737,9 +779,9 @@ static void x11_Blit( XMM_PluginGraph *graph )
   if( priv->xvd.image && priv->dyuv )
   {
 	XvShmPutImage( priv->display, priv->xvd.portID, priv->window, priv->gc,
-	    priv->xvd.image, 
-	    0, 0, priv->xvd.image->width, priv->xvd.image->height - priv->xvd.nvidia_driver,
-	    0, 0, priv->dwidth, priv->dheight, False );
+	    priv->xvd.image,
+	    0, 0, priv->xvd.image->width, priv->xvd.image->height,
+	    priv->dxoff, priv->dyoff, priv->dwidth, priv->dheight, False );
   }
   else
 #endif
@@ -779,7 +821,7 @@ XMM_PluginGraph	plugin_info = {	{ NULL,
 				"",
 				"X11",
 				"Graph: X11",
-				"Copyright (c) 2000, 2001 by Arthur Kleer",
+				"Copyright (c) 1999 Arthur Kleer",
 				NULL, NULL },
 				x11_Init, x11_Close, x11_Control, x11_Start,
 				x11_Draw, x11_Blit };
@@ -806,19 +848,25 @@ static int x11_Resize( XMM_PluginGraph *graph, int width, int height, int flags
   if(( !width || !height ) && !flags )
   {
 	xmmMutex_Unlock( priv->mutex );
-	return xmm_SetError( graph->sys.xmm, XMM_ERR_UNKNOWN, "(X11) Wrong parameters: width = 0, height = 0 only with flags ( FULLSCREEN )" );
+	return xmm_SetError( graph->sys.xmm, XMM_RET_ERROR, "(X11) Wrong parameters: width = 0, height = 0 only with flags ( FULLSCREEN )" );
   }
 
   if(( flags & XMM_GRAPH_RESIZE_ORIG ) == XMM_GRAPH_RESIZE_ORIG )
   {
 	width = priv->swidth;
 	height = priv->sheight;
+
+	priv->dxoff = 0;
+	priv->dyoff = 0;
   }
 
   if(( flags & XMM_GRAPH_RESIZE_DOUBLE ) == XMM_GRAPH_RESIZE_DOUBLE )
   {
 	width = priv->swidth << 1;
 	height = priv->sheight << 1;
+
+	priv->dxoff = 0;
+	priv->dyoff = 0;
   }
 
   /* Manage fullscreen */
@@ -827,9 +875,12 @@ static int x11_Resize( XMM_PluginGraph *graph, int width, int height, int flags
 	if( priv->Fullscreen )
 	{
 		xmmMutex_Unlock( priv->mutex );
-		return xmm_SetError( graph->sys.xmm, XMM_ERR_UNKNOWN, "(X11) Already fullscreen." );
+		return xmm_SetError( graph->sys.xmm, XMM_RET_ERROR, "(X11) Already fullscreen." );
 	}
 
+	priv->dxoff = 0;
+	priv->dyoff = 0;
+
 	/* Stretch to full screen ? */
 	if( !( width && height ))
 	{
@@ -889,7 +940,9 @@ static int x11_Resize( XMM_PluginGraph *graph, int width, int height, int flags
 	if( stretch )
 	{
 		width = priv->wwidth;
-		height = priv->wheight;
+		height = priv->sheight * width / priv->swidth;
+
+		priv->dyoff = ( priv->wheight - height ) >> 1;
 	}
 
 	/* framebuffer width correction */
@@ -955,8 +1008,8 @@ static int x11_Resize( XMM_PluginGraph *graph, int width, int height, int flags
   hints.flags = ( PSize | PMinSize | PMaxSize | PPosition );
   hints.x = 0;
   hints.y = 0;
-  hints.min_width = hints.max_width = hints.width = priv->dwidth;
-  hints.min_height = hints.max_height = hints.width = priv->dheight;
+  hints.min_width = hints.max_width = hints.width = priv->wwidth;
+  hints.min_height = hints.max_height = hints.width = priv->wheight;
 
 #if 0
   XSetWMNormalHints( priv->display, priv->window, &hints );
@@ -964,7 +1017,7 @@ static int x11_Resize( XMM_PluginGraph *graph, int width, int height, int flags
   XSetStandardProperties( priv->display, priv->window, XMM_PROJECT_NAME, XMM_PROJECT_NAME, None, NULL, 0, &hints );
 
   /* Resize window */
-  XResizeWindow( priv->display, priv->window, priv->dwidth, priv->dheight );
+  XResizeWindow( priv->display, priv->window, priv->wwidth, priv->wheight );
   XSync( priv->display, True);
   XMapWindow( priv->display, priv->window );
 
@@ -993,8 +1046,6 @@ static int x11_Resize( XMM_PluginGraph *graph, int width, int height, int flags
 	    if( xv_start( priv->display, &priv->xvd, priv->dformat, priv->swidth, priv->sheight ) >= 0 )
 	    {
 		xmm_logging( 2, "X11! Overlay (type: %s) created using XVideo extension.\n", xmm_FOURCC_string( priv->dformat ));
-		if( priv->xvd.nvidia_driver )
-		    xmm_logging( 2, "X11! WARNING: NVidia driver detected. Not displaying bottom line.\n" );
 
 		/* Unlock surface */
 		xmmMutex_Unlock( priv->mutex );
@@ -1003,7 +1054,7 @@ static int x11_Resize( XMM_PluginGraph *graph, int width, int height, int flags
 	    else
 	    {
 		xmmMutex_Unlock( priv->mutex );
-		return xmm_SetError( graph->sys.xmm, XMM_ERR_UNKNOWN, "(X11) Error initializing XVideo." );
+		return xmm_SetError( graph->sys.xmm, XMM_RET_ERROR, "(X11) Error initializing XVideo." );
 	    }
 	}
   }
@@ -1016,7 +1067,7 @@ static int x11_Resize( XMM_PluginGraph *graph, int width, int height, int flags
 	if( priv->image )	xshm_Destroy( priv->display, &priv->ShmInfo[0] );
 
 	/* Create new one */
-	if(( priv->image = xshm_Create( priv->display, priv->visual, &priv->ShmInfo[0], priv->dwidth, priv->dheight, priv->dbpp )) != NULL )
+	if(( priv->image = xshm_Create( priv->display, priv->visual, &priv->ShmInfo[0], priv->wwidth, priv->wheight, priv->dbpp )) != NULL )
 	{
 	    priv->image_data = priv->ShmInfo[0].shmaddr;
 	    priv->image->data = priv->image_data;
@@ -1085,3 +1136,34 @@ static int x11_CheckForEvent( XMM_PluginGraph *graph )
 
   return 0;
 }
+
+/*
+ * Set screen saver timeout
+ */
+static int x11_screen_saver( Display *display, int timeout )
+{
+  int interval, prefer_blank, allow_exp, toback;
+#ifdef DEBUG
+  int to;
+#endif
+
+  XGetScreenSaver( display, &toback, &interval, &prefer_blank, &allow_exp );
+
+#ifdef DEBUG
+  xmm_logging( 1, "X11! ScreenSaver: timeout = %i ( wanted %i ), interval = %i, blank = %i exp = %i\n",
+			toback, timeout, interval, prefer_blank, allow_exp );
+#endif
+
+  if( toback != timeout )
+  {
+	XSetScreenSaver( display, timeout, interval, prefer_blank, allow_exp );
+
+#ifdef DEBUG
+	XGetScreenSaver( display, &to, &interval, &prefer_blank, &allow_exp );
+	xmm_logging( 1, "X11! ScreenSaver(new): timeout = %i interval = %i blank = %i exp = %i\n",
+			to, interval, prefer_blank, allow_exp );
+#endif
+  }
+
+  return toback;
+}
