@@ -1,6 +1,6 @@
 /*
  *  XMMP - LinuX MultiMedia Project ( www.frozenproductions.com )
- *  Copyright (c) 1999 - 2001 Arthur Kleer <kleer@frozenproductions.com>
+ *  Copyright (c) 1999 - 2002 Arthur Kleer <kleer@frozenproductions.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -50,8 +50,6 @@ typedef struct xv_data_s
     XvImage		*image;
     XShmSegmentInfo	shmInfo;
 
-    /* Bug in NVidia driver */
-    int			nvidia_driver;    
 } xv_data_t;
 
 #endif
