@@ -1,6 +1,6 @@
 /*
  *  XMMP - LinuX MultiMedia Project ( www.frozenproductions.com )
- *  Copyright (c) 1999 - 2001 Arthur Kleer <kleer@frozenproductions.com>
+ *  Copyright (c) 1999 - 2002 Arthur Kleer <kleer@frozenproductions.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,36 +20,50 @@
 /*
  * mikmod.c
  * MikMod input
- *
- * TODO:
  */
 
-#include <stdio.h>
 #include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <mikmod.h>
 
 #include <libxmm/xmmp.h>
-#include <libxmm/config.h>
 #include <libxmm/version.h>
 #include <libxmm/xmmctl.h>
 #include <libxmm/lpinput.h>
+#include <libxmm/lpio.h>
 #include <libxmm/error.h>
 #include <libxmm/util/utils.h>
-#include <libxmm/util/thread.h>
 #include <libxmm/util/config.h>
-#include "priv.h"
+
+#include <mikmod.h>
+
+/*
+ * Types
+ */
+
+struct priv_t
+{
+    XMM_AudioInfo		ai;
+    XMM_ClipInfo		ci;
+
+    char 			*filename, path[XMM_MAXPATHLEN];
+    MODULE			*module;
+    uint32_t			position;
+    int				byterate;
+
+    struct mikmod_reader
+    {
+	MREADER			sys;
+	XMM_PluginIO		*pIO;
+    } mreader;
+};
 
 /*
  * Global data
  */
 
 extern XMM_PluginInput		plugin_info;
-
 extern MIKMODAPI		MDRIVER drv_xmm;
 
-struct priv_t 			*mikmod_priv = NULL;
+static struct priv_t 		*mikmod_priv = NULL;
 
 /*
  * Config
@@ -60,32 +74,24 @@ static int	cfg_ipolate = 1, cfg_surround = 0;
 
 static XMM_ConfigBlock mikmod_config[] =
 {
-    { &cfg_rate, "mikmod_rate", XMM_CFG_TYPE_INT },
-    { &cfg_size, "mikmod_size", XMM_CFG_TYPE_INT },
-    { &cfg_chan, "mikmod_chan", XMM_CFG_TYPE_INT },
-    { &cfg_ipolate, "mikmod_interpolate", XMM_CFG_TYPE_BOOL },
-    { &cfg_surround, "mikmod_surround", XMM_CFG_TYPE_BOOL },
-    { NULL, "", 0 }
+    { &cfg_rate,	"samplerate",	XMM_CFG_TYPE_INT,	4, "Samplerate (44100/22050/11025)" },
+    { &cfg_size,	"samplesize",	XMM_CFG_TYPE_INT,	4, "Samplesize (8/16)" },
+    { &cfg_chan,	"channels",	XMM_CFG_TYPE_INT,	4, "Channels (1/2)" },
+    { &cfg_ipolate,	"interpolate",	XMM_CFG_TYPE_BOOL,	4, "Interpolate" },
+    { &cfg_surround,	"surround",	XMM_CFG_TYPE_BOOL,	4, "Surround" },
+    { NULL,		"",		0,			0, "" }
 };
 
-
 /*
  * Prototypes
  */
 
-static int	mikmod_loop_thread( void *arg );
-
 static int wrap_Seek( struct MREADER *_mreader, long offset, int whence );
 static long wrap_Tell( struct MREADER *_mreader );
 static int wrap_Read( struct MREADER *_mreader, void *ptr, size_t size );
 static int wrap_Get( struct MREADER *_mreader );
 static int wrap_Eof( struct MREADER *_mreader );
 
-#ifdef PROVIDE_GTK_WINDOWS
-static void mikmod_gtk_about( void );
-static int mikmod_gtk_configure( void );
-#endif
-
 /*
  * Initialize
  */
@@ -97,30 +103,22 @@ static XMM_PluginInput *mikmod_Init( void *xmm )
   /* Check for multiple use. This plugin ( and MikMod ) may only be used once */
   if( mikmod_priv )
   {
-	xmm_SetError( xmm, XMM_ERR_UNKNOWN, "(MikMod) MikMod already in use." );
+	xmm_SetError( xmm, XMM_RET_ERROR, "(MikMod) MikMod already in use." );
 	return NULL;
   }
 
   if(( input = xmm_memdup_x( &plugin_info, sizeof( XMM_PluginInput ), sizeof( struct priv_t ))) == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_ALLOC, "(MikMod) Unable to duplicate plugin_info" );
+	xmm_SetError( xmm, XMM_RET_ALLOC, "(MikMod) Unable to duplicate plugin_info" );
 	return NULL;
   }
 
-  /*
-   * These have to be initialized
-   */
   mikmod_priv = priv = (struct priv_t *) &input[1];
   input->sys.priv = (void *) priv;
   input->sys.xmm = xmm;
 
-  /* Initialize sound */
-  priv->pSound = xmm_SoundOpen( xmm, (char *)xmm_AcSoundPluginName( xmm, NULL ));
-  if( priv->pSound == NULL )
-  {
-	free( input );
-	return (void *)NULL;
-  }
+  /* Read configuration */
+  xmmCfg_BlockLoad( "config", "input-mikmod", mikmod_config );
 
   return input;
 }
@@ -133,12 +131,9 @@ static int mikmod_Open( XMM_PluginInput *input, char *filename, int flags )
   struct priv_t		*priv = input->sys.priv;
   char			*buffer, *ptr, cmd[100], *tmpfile;
 
-  /* Only direct out mode supported */
-  if( flags & XMM_INPUT_CF_MODE_READ )
-	return xmm_SetError( input->sys.xmm, XMM_ERR_UNKNOWN, "(MikMod) Only 'direct out' mode supported." );
-
-  /* Read configuration */
-  xmmCfg_BlockLoad( "config", "input-mikmod", mikmod_config );
+  /* Only read mode supported */
+  if( flags & XMM_INPUT_CF_MODE_DOUT )
+	return xmm_SetError( input->sys.xmm, XMM_RET_ERROR, "(MikMod) Only 'read' mode supported." );
 
   /* Initialize MikMod */
   MikMod_RegisterDriver( &drv_xmm );
@@ -153,11 +148,11 @@ static int mikmod_Open( XMM_PluginInput *input, char *filename, int flags )
   if( cfg_ipolate )	md_mode |= DMODE_INTERP;
 
   if( MikMod_Init( "" ))
-	return xmm_SetError( input->sys.xmm, XMM_ERR_UNKNOWN, "(MikMod) Cannot initialize mikmod: %s", MikMod_strerror( MikMod_errno ));
+	return xmm_SetError( input->sys.xmm, XMM_RET_ERROR, "(MikMod) Cannot initialize mikmod: %s", MikMod_strerror( MikMod_errno ));
 
   /* Check filename */
   ptr = strrchr( filename, '.' );
-  if( ptr == NULL )	return xmm_SetError( input->sys.xmm, XMM_ERR_UNKNOWN, "(MikMod) Invalid file type. Should never happen." );
+  if( ptr == NULL )	return xmm_SetError( input->sys.xmm, XMM_RET_ERROR, "(MikMod) Invalid file type. Should never happen." );
 
   /* Get uncompressed tracker module format file */
   tmpfile = xmm_gethome_filename( "tmp/mikmod_zip" );
@@ -166,23 +161,23 @@ static int mikmod_Open( XMM_PluginInput *input, char *filename, int flags )
   {
 	sprintf( cmd, "gzip -dc %s >>%s", filename, tmpfile );
 	system( cmd );
-	strcpy( priv->mikmod_path, tmpfile );
+	strcpy( priv->path, tmpfile );
   }
   else if( !strcasecmp( ptr, ".bz2" ))
   {
 	sprintf( cmd, "bzip2 -dc %s >>%s", filename, tmpfile );
 	system( cmd );
-	strcpy( priv->mikmod_path, tmpfile );
+	strcpy( priv->path, tmpfile );
   }
-  else	strcpy( priv->mikmod_path, filename );
+  else	strcpy( priv->path, filename );
 
   free( tmpfile );
 
   /* Save filename */
   buffer = strdup( filename );
   ptr = strrchr( buffer, '/' );
-  if( ptr )	priv->mikmod_filename = strdup( ptr + 1 );
-  else	priv->mikmod_filename = strdup( buffer );
+  if( ptr )	priv->filename = strdup( ptr + 1 );
+  else	priv->filename = strdup( buffer );
   free( buffer );
 
   /* Open file */
@@ -194,36 +189,41 @@ static int mikmod_Open( XMM_PluginInput *input, char *filename, int flags )
   priv->mreader.sys.Read = wrap_Read;
   priv->mreader.sys.Get = wrap_Get;
   priv->mreader.sys.Eof = wrap_Eof;
-  /* Load file */
-#if 0
-  priv->module = Player_Load( priv->mikmod_path, 64, 0 );
-#else
+
+  /* Load Player and start it */
   priv->module = Player_LoadGeneric( &priv->mreader.sys, 64, 0 );
-#endif
   if( priv->module == NULL )
-	return xmm_SetError( input->sys.xmm, XMM_ERR_UNKNOWN, "(MikMod) Error loading file: %s\nFilename: %s", MikMod_strerror( MikMod_errno ), priv->mikmod_filename );
+	return xmm_SetError( input->sys.xmm, XMM_RET_ERROR, "(MikMod) Error loading file: %s\nFilename: %s", MikMod_strerror( MikMod_errno ), priv->filename );
 
-  /* Save audio info */
-  priv->ai.format.samprate = md_mixfreq;
-  priv->ai.format.channels = 2;
-  priv->ai.format.format = XMM_SOUND_FMT_S16LE;
-  priv->ai.bitrate = priv->module->bpm * 1000;
+  Player_Start( priv->module );
+
+  /* Fill audio format struct */
+  priv->ai.fmt.format = XMM_AUDIO_FMT_S16LE;
+  priv->ai.fmt.samprate = md_mixfreq;
+  priv->ai.fmt.channels = 2;
+  priv->ai.fmt.bitrate = md_mixfreq * 4;
+  priv->ai.fmt.blockSize = 4;
+  priv->ai.fmt.extraSize = 0;
+  priv->ai.fmt.desc[0] = '\0';
   priv->ai.tSamples = 0;
-  priv->bps = priv->ai.format.channels * ( priv->ai.format.format & XMM_SOUND_MASK_SIZE ) / 8;
+  priv->ai.tSize = priv->mreader.pIO->Size( priv->mreader.pIO );
+  priv->ai.offset = 0;
 
-  /* Start sound */
-  if(( priv->abuffersize = priv->pSound->Start( priv->pSound, &priv->ai.format, 8192, 4 )) < 0 )
-  	return XMM_RET_ERROR;
+  /* Byte rate */
+  priv->byterate = priv->ai.fmt.samprate * priv->ai.fmt.blockSize;
 
-  /* Allocate buffer size */
-  if(( priv->abuffer = malloc( priv->abuffersize * priv->bps )) == NULL )
-  {
-	priv->pSound->Close( priv->pSound );
-	return xmm_SetError( input->sys.xmm, XMM_ERR_UNKNOWN, "(MikMod) audio buffer" );
-  }
-
-  /* Create thread */
-  priv->mikmod_thread = xmmThread_Create( mikmod_loop_thread, (void *)priv );
+  /* Clip Info */
+  priv->ci.name[0] = '\0';
+  priv->ci.author[0] = '\0';
+  priv->ci.album[0] = '\0';
+  strncpy( priv->ci.content, Player_LoadTitle( priv->path ), XMM_CIL_CONTENT - 1 );
+  priv->ci.content[XMM_CIL_CONTENT - 1] = '\0';
+  priv->ci.copyright[0] = '\0';
+  priv->ci.software[0] = '\0';
+  priv->ci.date[0] = '\0';
+  priv->ci.comment[0] = '\0';
+  priv->ci.size = priv->mreader.pIO->Size( priv->mreader.pIO );
+  priv->ci.playtime = (double)0.0;
 
   return XMM_RET_OK;
 }
@@ -231,7 +231,6 @@ static int mikmod_Open( XMM_PluginInput *input, char *filename, int flags )
 /*
  * Close
  */
-
 static int mikmod_Close( XMM_PluginInput *input )
 {
   struct priv_t	*priv = input->sys.priv;
@@ -240,17 +239,11 @@ static int mikmod_Close( XMM_PluginInput *input )
   Player_Free( priv->module );
   MikMod_Exit();
 
-  priv->exit_mikmod_thread = 1;
-  xmmThread_Wait( priv->mikmod_thread );
-
-  free( priv->mikmod_filename );
+  if( priv->filename )	free( priv->filename );
   priv->mreader.pIO->Close( priv->mreader.pIO );
 
-  if( priv->abuffer )	free( priv->abuffer );
-  priv->abuffer = NULL;
-  priv->pSound->Close( priv->pSound );
-
   mikmod_priv = NULL;
+
   free( input );
   return XMM_RET_OK;
 }
@@ -260,98 +253,106 @@ static int mikmod_Close( XMM_PluginInput *input )
  */
 static int mikmod_Control( XMM_PluginInput *input, uint32_t cmd, uint32_t param, void *data )
 {
-  struct priv_t	*priv = input->sys.priv;
+  struct priv_t		*priv = input->sys.priv;
+  XMM_AudioFormat	*format;
+  XMM_ConfigBlock	*cfg;
 
   switch( cmd )
   {
 	case XMM_CTLQUERY_GFORMAT:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
 
-	/* Query sound format */
-	case XMM_CTLQUERY_SFORMAT:
-		return XMM_CTLRET_NOTSUPPORTED;
+	case XMM_CTLQUERY_AFORMAT:			/* Query sound format */
+		format = (XMM_AudioFormat *)data;
+		if(( format->format == priv->ai.fmt.format ) &&
+		    ( format->samprate == priv->ai.fmt.samprate ) &&
+		    ( format->channels == priv->ai.fmt.channels ))	return XMM_CTLRET_TRUE;
+		return XMM_CTLRET_FALSE;
 
 	case XMM_CTLQUERY_YFLIP:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
+
+	case XMM_CTLQUERY_CONFIG:
+		return XMM_CTLRET_TRUE;
 
 	case XMM_CTLGET_GFORMAT:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
 
-	/* Get sound format */
-	case XMM_CTLGET_SFORMAT:
-		return XMM_CTLRET_NOTSUPPORTED;
+	case XMM_CTLGET_AFORMAT:			/* Get sound format */
+		memcpy( data, &priv->ai.fmt, sizeof( XMM_AudioFormat ));
+		return XMM_CTLRET_ARG;			/* Result in arg */
+
+	case XMM_CTLGET_AFORMAT_PTR:			/* Get sound format */
+		*((XMM_AudioFormat **) data) = &priv->ai.fmt;
+		return XMM_CTLRET_ARG;			/* Result in arg */
 
 	case XMM_CTLGET_VOLUME:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
 
-	/* Get capabilities */
-	case XMM_CTLGET_CAPS:
-		*((uint32_t *) data ) = XMM_INPUT_CF_MODE_DOUT | XMM_INPUT_CF_AUDIO;
+	case XMM_CTLGET_CAPS:				/* Get capabilities */
+		*((uint32_t *) data ) = XMM_INPUT_CF_MODE_READ | XMM_INPUT_CF_AUDIO;
 		return XMM_CTLRET_ARG;
 
+	/* Get config */
+	case XMM_CTLGET_CONFIG:
+		if( data == NULL )	return XMM_RET_INVALID_ARG;
+
+		if( input->sys.priv == NULL )
+		{
+			/* Read configuration */
+			xmmCfg_BlockLoad( "config", "input-mikmod", mikmod_config );
+		}
+
+		cfg = xmmCfg_Blockdup( mikmod_config );
+		if( cfg == NULL )	return XMM_RET_ERROR;
+
+		*((XMM_ConfigBlock **)data) = cfg;
+		return XMM_CTLRET_ARG;
+
+	/* Set config */
+	case XMM_CTLSET_CONFIG:
+		if( data == NULL )	return XMM_RET_INVALID_ARG;
+
+		xmmCfg_BlockSave( "config", "input-mikmod", (XMM_ConfigBlock *)data );
+		return XMM_CTLRET_TRUE;
+
 	case XMM_CTLSET_GFORMAT:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
 
 	case XMM_CTLSET_SCALE:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
 
 	case XMM_CTLSET_VOLUME:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
 
 	/* Direct out mode */
 	case XMM_CTLINP_PLAY:
-		Player_Start( priv->module );
-		return XMM_CTLRET_TRUE;
-
 	case XMM_CTLINP_STOP:
-		Player_Stop();
-		return XMM_CTLRET_TRUE;
-
 	case XMM_CTLINP_PAUSE:
-		Player_TogglePause();
-		return XMM_CTLRET_TRUE;
-
 	case XMM_CTLINP_STATUS:
-		if( Player_Active())	*((uint32_t *) data ) = XMM_PBS_PLAYING;
-		else	*((uint32_t *) data ) = XMM_PBS_STOPPED;
-		return XMM_CTLRET_ARG;
+		return XMM_RET_NOTSUPPORTED;
 
 	/* Dialogues */
 	case XMM_CTLDLG_QUERY:
-#ifdef PROVIDE_GTK_WINDOWS
-		if(( param == XMM_GTKDLG_ABOUT ) ||
-		    ( param == XMM_GTKDLG_CONFIG ))	return XMM_CTLRET_TRUE;
-#endif
 		return XMM_CTLRET_FALSE;
 
 	case XMM_CTLDLG_DISPLAY:
-		switch( param )
-		{
-#ifdef PROVIDE_GTK_WINDOWS
-		    case XMM_GTKDLG_ABOUT:
-			    mikmod_gtk_about();
-			    break;
-
-		    case XMM_GTKDLG_CONFIG:
-			    mikmod_gtk_configure();
-			    break;
-#endif
-		    default:
-			    return XMM_CTLRET_NOTSUPPORTED;
-		}
+		return XMM_RET_NOTSUPPORTED;
 
 	default:
 		break;
   }
 
-  if( cmd & XMM_CTLMASK_INPUT )	return XMM_CTLRET_UNKNOWN;
-  return XMM_CTLRET_INVALID;	/* No INPUT command */
+  if( cmd & XMM_CTLMASK_INPUT )
+	return xmm_SetError( input->sys.xmm, XMM_RET_NOTSUPPORTED, "(MikMod) cmd = 0x%x" );
+
+  return xmm_SetError( input->sys.xmm, XMM_RET_INVALID_ARG, "(MikMod) cmd ( 0x%x )" );
 }
 
 /*
  * Seek to position
  */
-static int mikmod_Seek( XMM_PluginInput *input, double seek )
+static int mikmod_Seek( XMM_PluginInput *input, int vstream, int astream, double seek )
 {
   return XMM_RET_NOTSUPPORTED;
 }
@@ -359,31 +360,18 @@ static int mikmod_Seek( XMM_PluginInput *input, double seek )
 /*
  * Get Information
  */
-static double mikmod_Info( XMM_PluginInput *input, XMM_ClipInfo *ci, double *avdiff, double *seekval )
+static int mikmod_Info( XMM_PluginInput *input, XMM_ClipInfo *ci, double *seekval )
 {
   struct priv_t	*priv = input->sys.priv;
 
-  if( ci )
-  {
-	strncpy( ci->name, Player_LoadTitle( priv->mikmod_path ), 64 );
-	ci->name[63] = '\0';
-	ci->author[0] = '\0';
-	ci->copyright[0] = '\0';
-	ci->size = priv->mreader.pIO->Size( priv->mreader.pIO );
-	ci->playtime = (double)0.0;
-  }
+  if( ci )	memcpy( ci, &priv->ci, sizeof( XMM_ClipInfo ));
+  if( seekval )	*seekval = (double)priv->module->sngpos / priv->module->numpos;
 
-  if( avdiff )		*avdiff = (double)0.0;
-
-  if( seekval )		*seekval = (double)priv->module->sngpos / priv->module->numpos;
-
-  return (double)priv->module->sngtime / 1024;
+  return XMM_RET_OK;
 }
 
 /*
  * Get audio stream number
- * MikMod plugin does not support AudioRead. We still returned 1,
- * so information can be obtained using AudioInfo()
  */
 static int mikmod_AudioStreams( XMM_PluginInput *input )
 {
@@ -393,12 +381,11 @@ static int mikmod_AudioStreams( XMM_PluginInput *input )
 /*
  * Get audio stream information
  */
-static int mikmod_AudioInfo( XMM_PluginInput *input, int stream, XMM_AudioInfo *ai, uint32_t *cSample )
+static int mikmod_AudioInfo( XMM_PluginInput *input, int stream, XMM_AudioInfo *ai )
 {
   struct priv_t	*priv = input->sys.priv;
 
   if( ai )	memcpy( ai, &priv->ai, sizeof( XMM_AudioInfo ));
-  if( cSample )	*cSample = 0;
 
   return XMM_RET_OK;
 }
@@ -406,15 +393,33 @@ static int mikmod_AudioInfo( XMM_PluginInput *input, int stream, XMM_AudioInfo *
 /*
  * Read audio data
  */
-static int mikmod_AudioRead( XMM_PluginInput *input, int stream, uint8_t *buffer, uint32_t samples )
+static int mikmod_AudioRead( XMM_PluginInput *input, int stream, uint8_t *buffer, uint32_t size, int flags )
 {
-  return XMM_RET_NOTSUPPORTED;
+  struct priv_t	*priv = input->sys.priv;
+  uint32_t	read = 0;
+
+  read = VC_WriteBytes( (SBYTE *) buffer, size );
+  priv->position += read;
+
+  return read;
+}
+
+/*
+ * Audio timestamp.
+ */
+static int mikmod_AudioPTS( XMM_PluginInput *input, int stream, uint32_t *pts )
+{
+  struct priv_t	*priv = input->sys.priv;
+
+  if( pts )	*pts = (uint32_t)((double)priv->position / priv->byterate * 1000 );
+
+  return XMM_RET_OK;
 }
 
 /*
  * Seek to position in audio stream
  */
-static int mikmod_AudioSeek( XMM_PluginInput *input, int stream, uint32_t sample )
+static int mikmod_AudioSeek( XMM_PluginInput *input, int stream, uint32_t pts )
 {
   return XMM_RET_NOTSUPPORTED;
 }
@@ -432,7 +437,7 @@ static int mikmod_VideoInfo( XMM_PluginInput *input, int stream, XMM_VideoInfo *
   return XMM_RET_NOTSUPPORTED;
 }
 
-static int mikmod_VideoRead( XMM_PluginInput *input, int stream, uint8_t *buffer[] )
+static int mikmod_VideoRead( XMM_PluginInput *input, int stream, uint8_t *buffer[], int flags )
 {
   return XMM_RET_NOTSUPPORTED;
 }
@@ -496,7 +501,6 @@ static int mikmod_FileInfo( void *xmm, char *filename, XMM_ClipInfo *ci )
   return XMM_RET_NOTSUPPORTED;
 }
 
-
 /*
  * Plugin data
  */
@@ -509,12 +513,13 @@ XMM_PluginInput	plugin_info = {	{ NULL,
 				"",
 				"MikMod",
 				"Input: MikMod",
-				"Copyright (c) 2000, 2001 by Arthur Kleer",
+				"Copyright (c) 2000 Arthur Kleer",
 				NULL, NULL },
 				mikmod_Init, mikmod_Open, mikmod_Close,
 				mikmod_Control, mikmod_Seek, mikmod_Info, 
 				mikmod_AudioStreams, mikmod_AudioInfo,
-				mikmod_AudioRead, mikmod_AudioSeek,
+				mikmod_AudioRead, mikmod_AudioPTS,
+				mikmod_AudioSeek,
 				mikmod_VideoStreams, mikmod_VideoInfo,
 				mikmod_VideoRead, mikmod_VideoPTS,
 				mikmod_VideoSeek,
@@ -525,25 +530,66 @@ XMM_PluginInput	plugin_info = {	{ NULL,
  */
 
 /*
- * Player thread
+ * MikMod driver
  */
 
-static int mikmod_loop_thread( void *arg )
+/*
+ * Functions
+ */
+
+static BOOL xmm_IsPresent( void )
 {
-  struct priv_t	*priv = arg;
-
-  priv->exit_mikmod_thread = 0;
-
-  do
-  {
-	if( Player_Active())	drv_xmm.Update();
-	else	xmmSYS_usleep( 10000 );
-
-  }  while( !priv->exit_mikmod_thread );
-
-  return XMM_RET_OK;
+  return 1;
 }
 
+static BOOL xmm_Reset( void )
+{
+  VC_Exit();
+  return VC_Init();
+}
+
+static void xmm_Update( void )
+{
+}
+
+/*
+ * Driver data
+ */
+
+MIKMODAPI MDRIVER drv_xmm = {
+	NULL,
+	"xmm",
+	"xmm output driver v1.0",
+	0,
+	255,
+	"xmm",
+	NULL,
+	xmm_IsPresent,
+	VC_SampleLoad,
+	VC_SampleUnload,
+	VC_SampleSpace,
+	VC_SampleLength,
+	VC_Init,
+	VC_Exit,
+	xmm_Reset,
+	VC_SetNumVoices,
+	VC_PlayStart,
+	VC_PlayStop,
+	xmm_Update,
+	NULL,
+	VC_VoiceSetVolume,
+	VC_VoiceGetVolume,
+	VC_VoiceSetFrequency,
+	VC_VoiceGetFrequency,
+	VC_VoiceSetPanning,
+	VC_VoiceGetPanning,
+	VC_VoicePlay,
+	VC_VoiceStop,
+	VC_VoiceStopped,
+	VC_VoiceGetPosition,
+	VC_VoiceRealVolume
+};
+
 /*
  * MREADER wrapper functions
  */
@@ -601,120 +647,3 @@ static int wrap_Eof( struct MREADER *_mreader )
 
   return ret;
 }
-
-/* ========================================================================== *
- * * * * * * * * * * * * * * * * * * Gtk Stuff  * * * * * * * * * * * * * * * *
- * ========================================================================== */
-
-#ifdef PROVIDE_GTK_WINDOWS
-
-#include <gtk/gtk.h>
-#include "gtk/interface.h"
-#include "gtk/support.h"
-
-/*
- * About
- */
-
-static void mikmod_gtk_about( void )
-{
-  GtkWidget *about_window;
-
-  about_window = create_mikmod_about_window();
-  gtk_widget_show( about_window );
-}
-
-/*
- * Configure
- */
-
-/*
- * Callback for settings OK button
- */
-
-void on_mikmod_pref_ok_clicked( GtkWidget *pref_window, gpointer user_data )
-{
-  if( GTK_TOGGLE_BUTTON( lookup_widget( pref_window,
-		    "pref_quality_44khz" ))->active )	cfg_rate = 44100;
-  if( GTK_TOGGLE_BUTTON( lookup_widget( pref_window,
-		    "pref_quality_22khz" ))->active )	cfg_rate = 22050;
-  if( GTK_TOGGLE_BUTTON( lookup_widget( pref_window,
-		    "pref_quality_11khz" ))->active )	cfg_rate = 11025;
-
-  if( GTK_TOGGLE_BUTTON( lookup_widget( pref_window,
- 		    "pref_quality_stereo" ))->active )	cfg_chan = 2;
-  if( GTK_TOGGLE_BUTTON( lookup_widget( pref_window,
-		    "pref_quality_mono" ))->active )	cfg_chan = 1;
-
-  if( GTK_TOGGLE_BUTTON( lookup_widget( pref_window,
-		    "pref_quality_16bit" ))->active )	cfg_size = 16;
-  if( GTK_TOGGLE_BUTTON( lookup_widget( pref_window,
-		    "pref_quality_8bit" ))->active )	cfg_size = 8;
-
-  if( GTK_TOGGLE_BUTTON( lookup_widget( pref_window,
-		    "pref_ipolate" ))->active )		cfg_ipolate = 1;
-  else	cfg_ipolate = 0;
-
-  if( GTK_TOGGLE_BUTTON( lookup_widget( pref_window,
-		    "pref_surround" ))->active )	cfg_surround = 1;
-  else	cfg_surround = 0;
-
-  xmmCfg_BlockSave( "config", "input-mikmod", mikmod_config );
-
-  gtk_widget_destroy( pref_window );
-}
-
-/*
- * Create window
- */
-
-static int mikmod_gtk_configure( void )
-{
-  GtkWidget	*pref_window;
-
-  /* Create window */
-  pref_window = create_mikmod_pref_window();
-
-  /* Load config */
-  xmmCfg_BlockLoad( "config", "input-mikmod", mikmod_config );
-
-  /*
-   * Set widgets
-   */
-
-  if( cfg_rate == 44100 )
-	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lookup_widget(
-			    pref_window, "pref_quality_44khz" )), TRUE );
-  if( cfg_rate == 22050 )
-	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lookup_widget(
-			    pref_window, "pref_quality_22khz" )), TRUE );
-  if( cfg_rate == 11025 )
-	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lookup_widget(
-			    pref_window, "pref_quality_11khz" )), TRUE );
-
-  if( cfg_chan == 2 )
-	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lookup_widget(
-			    pref_window, "pref_quality_stereo" )), TRUE );
-  if( cfg_chan == 1 )
-	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lookup_widget(
-			    pref_window, "pref_quality_mono" )), TRUE );
-
-  if( cfg_size == 8 )
-	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lookup_widget(
-			    pref_window, "pref_quality_8bit" )), TRUE );
-  if( cfg_size == 16 )
-	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lookup_widget(
-			    pref_window, "pref_quality_16bit" )), TRUE );
-
-  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lookup_widget(
-			    pref_window, "pref_ipolate" )), cfg_ipolate );
-
-  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( lookup_widget(
-			    pref_window, "pref_surround" )), cfg_surround );
-
-  gtk_widget_show( pref_window );
-
-  return 0;
-}
-
-#endif
