@@ -1,6 +1,6 @@
 /*
  *  XMMP - LinuX MultiMedia Project ( www.frozenproductions.com )
- *  Copyright (c) 1999 - 2001 Arthur Kleer <kleer@frozenproductions.com>
+ *  Copyright (c) 1999 - 2002 Arthur Kleer <kleer@frozenproductions.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,10 +19,7 @@
 
 /*
  * wave.c
- * WAVE input
- *
- * TODO:
- *  - AudioSeek() only seeks to sample positions multiple of the block size
+ * WAVE file format DeMUX
  */
 
 #include <stdio.h>
@@ -33,7 +30,6 @@
 #include <libxmm/version.h>
 #include <libxmm/xmmctl.h>
 #include <libxmm/lpinput.h>
-#include <libxmm/lpcodeca.h>
 #include <libxmm/lpio.h>
 #include <libxmm/error.h>
 #include <libxmm/util/utils.h>
@@ -42,6 +38,11 @@
 /*
  * Definitions
  */
+
+/* Change 'undef' in 'define' to get verbose info */
+#ifndef VERBOSE
+#define	VERBOSE
+#endif
  
 #define alignUP( a,b )	((((a) + (b) - 1 ) / (b)) * (b))
 
@@ -51,16 +52,17 @@
 
 struct priv_t
 {
-    XMM_PluginCodecAudio	*pCodec;
     XMM_PluginIO		*pIO;
     XMM_AudioInfo		ai;
+    XMM_AudioFormat		*af;
+
+    /* Clip info */
+    XMM_ClipInfo		ci;
 
-    uint32_t			cSample, data_offset;
-    int				seekable, bps, blockSize;
     char 			*filename;
-
-    char			*rbuffer;
-    int				rbuffersize;
+    int				seekable;
+    uint32_t			data_offset, data_size;
+    int				byterate;
 };
 
 /*
@@ -78,9 +80,12 @@ static uint32_t	READ32( XMM_PluginIO *wave );
 static uint16_t	READ16( XMM_PluginIO *wave );
 static uint8_t	READ8( XMM_PluginIO *wave );
 static int	SEEK( XMM_PluginIO *wave, long offset, int seekable );
-static int	Read_fmt( XMM_PluginIO *wave, WAVEFORMATEX *wave_fmt_head, uint32_t size, unsigned char **wave_fmt_extra );
 
-static int wave_AudioSeek( XMM_PluginInput *input, int stream, uint32_t sample );
+static int	Read_fmt( XMM_PluginIO *wave, WAVEFORMATEX *wave_fmt_head, uint32_t size, unsigned char **wave_fmt_extra );
+static int	Read_INFO_chunk( XMM_PluginIO *pIO, XMM_ClipInfo *ci, uint32_t dwName, uint32_t dwSize );
+
+static int	wave_AudioSeek( XMM_PluginInput *input, int stream, uint32_t sample );
+uint32_t	codec_ms2xmm( uint16_t ms_codec );
 
 /*
  * Initialize
@@ -92,7 +97,7 @@ static XMM_PluginInput *wave_Init( void *xmm )
 
   if(( input = xmm_memdup_x( &plugin_info, sizeof( XMM_PluginInput ), sizeof( struct priv_t ))) == NULL )
   {
-	xmm_SetError( xmm, XMM_ERR_ALLOC, "(WAVE) Unable to duplicate plugin_info" );
+	xmm_SetError( xmm, XMM_RET_ALLOC, "(WAVE) plugin_info" );
 	return NULL;
   }
 
@@ -109,16 +114,15 @@ static XMM_PluginInput *wave_Init( void *xmm )
 static int wave_Open( XMM_PluginInput *input, char *filename, int flags )
 {
   struct priv_t		*priv = input->sys.priv;
-  uint32_t		dwSize, dwName, dwTemp, data_size = 0;
+  uint32_t		dwSize, dwName, dwTemp;
   char			*buffer, *ptr;
   int			loop = 1;
-  XMM_AudioFormat	*af;
   WAVEFORMATEX		wave_fmt_head;
   unsigned char		*wave_fmt_extra;
 
   /* Only read mode supported */
   if( flags & XMM_INPUT_CF_MODE_DOUT )
-	return xmm_SetError( input->sys.xmm, XMM_ERR_UNKNOWN, "(WAVE) Only 'read' mode supported" );
+	return xmm_SetError( input->sys.xmm, XMM_RET_ERROR, "(WAVE) Only 'read' mode supported" );
 
   /* Open file */
   priv->pIO = xmm_IOOpen( input->sys.xmm, filename, XMM_IO_READ );
@@ -128,7 +132,16 @@ static int wave_Open( XMM_PluginInput *input, char *filename, int flags )
   if( priv->pIO->Seek( priv->pIO, 0, XMM_SEEK_CUR ) == XMM_RET_OK )
 	priv->seekable = 1;
   else	priv->seekable = 0;
-  xmm_logging( 2, "WAVE! Stream is %s seekable\n", priv->seekable ? "" : "NOT" );
+#ifdef VERBOSE
+  xmm_logging( 2, "WAVE! Stream is %sseekable\n", priv->seekable ? "" : "NOT " );
+#endif
+
+  /* Clip info */
+  memset( &priv->ci, 0, sizeof( XMM_ClipInfo ));
+
+  /* Init data */
+  priv->data_offset = 0;
+  priv->data_size = 0;
 
   /* Parse file */
   while( loop )
@@ -136,10 +149,29 @@ static int wave_Open( XMM_PluginInput *input, char *filename, int flags )
 	dwName = READ32( priv->pIO );
 	if( priv->pIO->Eof( priv->pIO ))	break;
 
+#ifdef VERBOSE
 	xmm_logging( 2, "WAVE! Chunk ID = '%s' ( %lx )\n", xmm_FOURCC_string( dwName ), dwName );
+#endif
 
-	dwSize = READ32( priv->pIO );
-	xmm_logging( 2, "WAVE! Chunk Size = %li ( %lx )\n", dwSize, dwSize );
+	/*
+	 * Handle list chunks
+	 */
+	switch( dwName )
+	{
+		case RIFF_ID_INFO:
+			dwName = 0;
+			break;
+
+		/* Non-list chunk: read size */
+		default:
+			dwSize = READ32( priv->pIO );
+#ifdef VERBOSE
+			xmm_logging( 2, "WAVE! ID = %x [%s] Chunk Size = %li ( %lx )\n", dwName, xmm_FOURCC_string(dwName), dwSize, dwSize );
+#endif
+			break;
+	}
+
+	if( dwName == 0 )	continue;
 	
 	switch( dwName )
 	{
@@ -148,24 +180,37 @@ static int wave_Open( XMM_PluginInput *input, char *filename, int flags )
 			if( dwTemp != RIFF_ID_WAVE )
 			{
 				priv->pIO->Close( priv->pIO );
-				return xmm_SetError( input->sys.xmm, XMM_ERR_UNKNOWN, "(WAVE) No WAVE RIFF form ( '%s' )", xmm_FOURCC_string( dwTemp ));
+				return xmm_SetError( input->sys.xmm, XMM_RET_ERROR, "(WAVE) No WAVE RIFF form ( '%s' )", xmm_FOURCC_string( dwTemp ));
 			}
 			xmm_logging( 2, "WAVE! RIFF Type = '%s'\n", xmm_FOURCC_string( dwTemp ));
 			break;
 
+		case RIFF_ID_LIST:
+			break;
+
 		case RIFF_ID_fmt:
 			Read_fmt( priv->pIO, &wave_fmt_head, dwSize, &wave_fmt_extra );
 			break;
 
 		case RIFF_ID_data:
 			priv->data_offset = priv->pIO->Tell( priv->pIO );
-			data_size = dwSize;
-			xmm_logging( 2, "WAVE! data_offset = %li ( %lx )\n", priv->data_offset, priv->data_offset );
+			priv->data_size = dwSize;
+			xmm_logging( 2, "WAVE! data: offset = %li ( %lx ) size = %li ( %lx )\n", priv->data_offset, priv->data_offset, priv->data_size, priv->data_size );
 			if( dwSize & 1 )	dwSize++;
 			if( priv->seekable )	SEEK( priv->pIO, dwSize, priv->seekable );
 			else	loop = 0;
 			break;
 
+		/* INFO chunks */
+		case RIFF_ID_INAM:
+		case RIFF_ID_ISBJ:
+		case RIFF_ID_IART:
+		case RIFF_ID_ICOP:
+		case RIFF_ID_ISFT:
+		case RIFF_ID_ICMT:
+			Read_INFO_chunk( priv->pIO, &priv->ci, dwName, dwSize );
+			break;
+
 		default:
 			xmm_logging( 2, "WAVE! Ignoring chunk '%s' (%lx)\n", xmm_FOURCC_string(dwName), dwName );
 			if( dwSize & 1 )	dwSize++;
@@ -174,6 +219,15 @@ static int wave_Open( XMM_PluginInput *input, char *filename, int flags )
 	}
   }
 
+  if( priv->data_offset == 0 )
+  {
+	priv->pIO->Close( priv->pIO );
+	return xmm_SetError( input->sys.xmm, XMM_RET_ERROR, "(WAVE) No data found" );
+  }
+
+  /* Seek to the beginning of data */
+  if( priv->seekable )	priv->pIO->Seek( priv->pIO, priv->data_offset, XMM_SEEK_SET );
+
   /* Save filename */
   buffer = strdup( filename );
   ptr = strrchr( buffer, '/' );
@@ -181,43 +235,50 @@ static int wave_Open( XMM_PluginInput *input, char *filename, int flags )
   else	priv->filename = strdup( buffer );
   free( buffer );
 
-  /* Set input data */
-  if(( af = malloc( sizeof( XMM_AudioFormat ) + wave_fmt_head.cbSize )) == NULL )
+  /* Allocate memory */
+  if(( priv->af = malloc( sizeof( XMM_AudioFormat ) + wave_fmt_head.cbSize )) == NULL )
+	return xmm_SetError( input->sys.xmm, XMM_RET_ALLOC, "(WAVE) XMM_AudioFormat" );
+
+  /* Fill audio format struct */
+  priv->af->format = codec_ms2xmm( wave_fmt_head.wFormatTag );
+  priv->af->format |= ( wave_fmt_head.wBitsPerSample & XMM_AUDIO_MASK_SIZE );
+
+  if(( wave_fmt_head.wFormatTag == WAVE_FORMAT_IEEE_FLOAT ) ||
+	(( wave_fmt_head.wBitsPerSample != 8 )))
+		priv->af->format |= XMM_AUDIO_MASK_SIGNED;
+
+  priv->af->samprate = wave_fmt_head.nSamplesPerSec;
+  priv->af->channels = wave_fmt_head.nChannels;
+  priv->af->bitrate = wave_fmt_head.nAvgBytesPerSec * 8;
+  priv->af->blockSize = wave_fmt_head.nBlockAlign;
+  priv->af->extraSize = wave_fmt_head.cbSize;
+  priv->af->extraType = XMM_AUDIO_EXT_WAVE;
+  priv->af->desc[0] = '\0';
+
+  if( priv->af->extraSize )
+	memcpy( &priv->af[1], wave_fmt_extra, priv->af->extraSize );
+
+  /* AudioInfo struct */
+  memcpy( &priv->ai.fmt, priv->af, sizeof( XMM_AudioFormat ));
+  priv->ai.fmt.extraSize = 0;
+  priv->ai.tSamples = (uint32_t)(((double)priv->data_size /
+			    ( priv->af->bitrate / 8 )) * priv->af->samprate );
+  priv->ai.tSize = priv->data_size;
+  priv->ai.offset = 0;
+
+  /* Byte rate */
+  priv->byterate = priv->ai.fmt.bitrate / 8;
+
+  /* Clip info */
+  priv->ci.size = priv->pIO->Size( priv->pIO );
+  priv->ci.playtime = (double)priv->ai.tSamples / priv->ai.fmt.samprate;
+
+  if( priv->ci.content[0] == '\0' )
   {
-	return xmm_SetError( input->sys.xmm, XMM_ERR_ALLOC, "(WAVE) acm_format_source" );
+	strncpy( priv->ci.content, priv->filename, XMM_CIL_CONTENT - 1 );
+	priv->ci.content[XMM_CIL_CONTENT - 1] = '\0';
   }
 
-  af->formatID = wave_fmt_head.wFormatTag;
-  af->channels = wave_fmt_head.nChannels;
-  af->samprate = wave_fmt_head.nSamplesPerSec;
-  af->bitrate = wave_fmt_head.nAvgBytesPerSec * 8;
-  af->format = (( wave_fmt_head.wBitsPerSample == 8 ) ?
-				XMM_SOUND_FMT_U8 : XMM_SOUND_FMT_S16LE );
-  af->blockSize = wave_fmt_head.nBlockAlign;
-  af->extraSize = wave_fmt_head.cbSize;
-  af->desc[0] = '\0';
-
-  memcpy( &af[1], wave_fmt_extra, af->extraSize );
-
-  /* Initialize audio codec */
-  priv->pCodec = xmm_CodecAudioOpen( input->sys.xmm, XMM_CODEC_MODE_DECODE, af );
-  if( priv->pCodec == NULL )	return XMM_RET_ERROR;
-
-  /* Query codec output format */
-  priv->pCodec->Control( priv->pCodec, XMM_CTLGET_SFORMAT, &priv->ai.format );
-
-  /* Seek to the beginning of data */
-  if( priv->seekable )	priv->pIO->Seek( priv->pIO, priv->data_offset, XMM_SEEK_SET );
-
-  /* Initialize data */
-  priv->rbuffersize = -1;
-  priv->cSample = 0;
-  priv->bps = priv->ai.format.channels * ( priv->ai.format.format & XMM_SOUND_MASK_SIZE ) / 8;
-  priv->blockSize = af->blockSize;
-  priv->ai.bitrate = af->bitrate;
-  priv->ai.tSamples = (uint32_t)(((double)data_size / ( af->bitrate / 8 )) *
-						priv->ai.format.samprate );
-
   return XMM_RET_OK;
 }
 
@@ -230,10 +291,9 @@ static int wave_Close( XMM_PluginInput *input )
 
   if( priv->filename )	free( priv->filename );
   if( priv->pIO )	priv->pIO->Close( priv->pIO );
-  if( priv->rbuffer )	free( priv->rbuffer );
-  if( priv->pCodec )	priv->pCodec->Close( priv->pCodec );
-
+  if( priv->af )	free( priv->af );
   free( input );
+
   return XMM_RET_OK;
 }
 
@@ -243,114 +303,101 @@ static int wave_Close( XMM_PluginInput *input )
 static int wave_Control( XMM_PluginInput *input, uint32_t cmd, uint32_t param, void *data )
 {
   struct priv_t		*priv = input->sys.priv;
-  XMM_SoundFormat	*format;
+  XMM_AudioFormat	*format;
 
   switch( cmd )
   {
 	case XMM_CTLQUERY_GFORMAT:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
+
+	case XMM_CTLQUERY_AFORMAT:			/* Query sound format */
+		format = (XMM_AudioFormat *)data;
+		if(( format->format == priv->ai.fmt.format ) &&
+		    ( format->samprate == priv->ai.fmt.samprate ) &&
+		    ( format->channels == priv->ai.fmt.channels ))
+			return XMM_CTLRET_TRUE;
 
-	/* Query sound format */
-	case XMM_CTLQUERY_SFORMAT:
-		format = (XMM_SoundFormat *)data;
-		if(( format->format == priv->ai.format.format ) &&
-		    ( format->samprate = priv->ai.format.samprate ) &&
-		    ( format->channels == priv->ai.format.channels ))	return XMM_CTLRET_TRUE;
 		return XMM_CTLRET_FALSE;
 
 	case XMM_CTLQUERY_YFLIP:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
+
+	case XMM_CTLQUERY_CONFIG:
+		return XMM_CTLRET_FALSE;
 
 	case XMM_CTLGET_GFORMAT:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
 
-	/* Get sound format */
-	case XMM_CTLGET_SFORMAT:
-		format = (XMM_SoundFormat *)data;
-		format->channels = priv->ai.format.channels;
-		format->samprate = priv->ai.format.samprate;
-		format->format = priv->ai.format.format;
+	case XMM_CTLGET_AFORMAT:			/* Get sound format */
+		memcpy( data, priv->af, sizeof( XMM_AudioFormat ));
+		((XMM_AudioFormat *)data)->extraSize = 0;
+		return XMM_CTLRET_ARG;			/* Result in arg */
+
+	case XMM_CTLGET_AFORMAT_PTR:			/* Get sound format */
+		*((XMM_AudioFormat **) data) = priv->af;
 		return XMM_CTLRET_ARG;			/* Result in arg */
 
 	case XMM_CTLGET_VOLUME:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
 
-	/* Get capabilities */
-	case XMM_CTLGET_CAPS:
-		*((uint32_t *) data ) = XMM_INPUT_CF_MODE_READ | XMM_INPUT_CF_AUDIO;
+	case XMM_CTLGET_CAPS:				/* Get capabilities */
+		*((uint32_t *) data ) = XMM_INPUT_CF_MODE_READ |
+					XMM_INPUT_CF_AUDIO;
 		return XMM_CTLRET_ARG;
 
 	case XMM_CTLSET_GFORMAT:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
 
 	case XMM_CTLSET_SCALE:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
 
 	case XMM_CTLSET_VOLUME:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
 
 	/* Direct out mode */
 	case XMM_CTLINP_PLAY:
 	case XMM_CTLINP_STOP:
 	case XMM_CTLINP_PAUSE:
 	case XMM_CTLINP_STATUS:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
 
 	/* Dialogues */
 	case XMM_CTLDLG_QUERY:
 		return XMM_CTLRET_FALSE;
 
 	case XMM_CTLDLG_DISPLAY:
-		return XMM_CTLRET_NOTSUPPORTED;
+		return XMM_RET_NOTSUPPORTED;
 
 	default:
 		break;
   }
 
-  if( cmd & XMM_CTLMASK_INPUT )	return XMM_CTLRET_UNKNOWN;
-  return XMM_CTLRET_INVALID;	/* No INPUT command */
+  if( cmd & XMM_CTLMASK_INPUT )
+	return xmm_SetError( input->sys.xmm, XMM_RET_NOTSUPPORTED, "(WAVE) cmd = 0x%x" );
+
+  return xmm_SetError( input->sys.xmm, XMM_RET_INVALID_ARG, "(WAVE) cmd ( 0x%x )" );
 }
 
 /*
  * Seek to position
  */
-static int wave_Seek( XMM_PluginInput *input, double seek )
+static int wave_Seek( XMM_PluginInput *input, int vstream, int astream, double seek )
 {
   struct priv_t	*priv = input->sys.priv;
 
-  if( priv->seekable == 0 )
-	return xmm_SetError( input->sys.xmm, XMM_ERR_NOTSUPPORTED, "(WAVE) Stream is not seekable." );
-
-  /* Clip seek value */
-  if( seek < 0.0 )	seek = 0.0;
-  if( seek > 100.0 )	seek = 100.0;
-
-  /* Calculate sample from seek value */
-  return wave_AudioSeek( input, 0, (uint32_t)( seek * priv->ai.tSamples ));
+  return input->AudioSeek( input, astream, (uint32_t)( seek * priv->ai.tSize / priv->byterate * 1000 ));
 }
 
 /*
  * Get Information
  */
-static double wave_Info( XMM_PluginInput *input, XMM_ClipInfo *ci, double *avdiff, double *seekval )
+static int wave_Info( XMM_PluginInput *input, XMM_ClipInfo *ci, double *seekval )
 {
   struct priv_t	*priv = input->sys.priv;
 
-  if( ci )
-  {
-	strncpy( ci->name, priv->filename, 64 );
-	ci->name[63] = '\0';
-	ci->author[0] = '\0';
-	ci->copyright[0] = '\0';
-	ci->size = priv->pIO->Size( priv->pIO );
-	ci->playtime = (double)priv->ai.tSamples / priv->ai.format.samprate;
-  }
+  if( ci )	memcpy( ci, &priv->ci, sizeof( XMM_ClipInfo ));
 
-  if( avdiff )		*avdiff = (double)0.0;
-
-  if( seekval )		*seekval = (double)priv->cSample / priv->ai.tSamples;
-
-  return (double)priv->cSample / priv->ai.format.samprate;
+  return XMM_RET_OK;
 }
 
 /*
@@ -364,12 +411,11 @@ static int wave_AudioStreams( XMM_PluginInput *input )
 /*
  * Get audio stream information
  */
-static int wave_AudioInfo( XMM_PluginInput *input, int stream, XMM_AudioInfo *ai, uint32_t *cSample )
+static int wave_AudioInfo( XMM_PluginInput *input, int stream, XMM_AudioInfo *ai )
 {
   struct priv_t	*priv = input->sys.priv;
 
   if( ai )	memcpy( ai, &priv->ai, sizeof( XMM_AudioInfo ));
-  if( cSample )	*cSample = priv->cSample;
 
   return XMM_RET_OK;
 }
@@ -377,61 +423,51 @@ static int wave_AudioInfo( XMM_PluginInput *input, int stream, XMM_AudioInfo *ai
 /*
  * Read audio data
  */
-static int wave_AudioRead( XMM_PluginInput *input, int stream, uint8_t *buffer, uint32_t samples )
+static int wave_AudioRead( XMM_PluginInput *input, int stream, uint8_t *buffer, uint32_t size, int flags )
 {
   struct priv_t *priv = input->sys.priv;
-  uint32_t	read, decoded;
+  uint32_t	read;
 
-  decoded = read = samples * priv->bps;
-  priv->pCodec->Control( priv->pCodec, XMM_CTLGET_DATA_SSIZE, &read );
-
-  if(( priv->rbuffersize < read ) && priv->rbuffer )
+  if(( priv->pIO->Tell( priv->pIO ) + size ) >= ( priv->data_offset + priv->data_size ))
   {
-	free( priv->rbuffer );
-	priv->rbuffer = NULL;
+    size = priv->data_offset + priv->data_size - priv->pIO->Tell( priv->pIO );
+    if( size <= 0 )	return XMM_RET_EOS;
   }
 
-  if( priv->rbuffer == NULL )
-  {
-	priv->rbuffersize = read;
-	if(( priv->rbuffer = malloc( priv->rbuffersize )) == NULL )
-	{
-		return xmm_SetError( input->sys.xmm, XMM_ERR_ALLOC, "(WAVE) audio buffer." );
-	}
-  }
+  read = priv->pIO->Read( priv->pIO, buffer, 1, size );
+  if(( read == 0 ) && priv->pIO->Eof( priv->pIO ))	return XMM_RET_EOS;
 
-  read = priv->pIO->Read( priv->pIO, priv->rbuffer, 1, read );
-  if(( read == 0 ) && priv->pIO->Tell( priv->pIO ))	return XMM_RET_EOS;
+  return read;
+}
 
-  decoded = samples;
-  priv->pCodec->Decode( priv->pCodec, priv->rbuffer, read, buffer, &decoded, NULL );
-  priv->cSample += decoded;
+/*
+ * Audio timestamp.
+ */
+static int wave_AudioPTS( XMM_PluginInput *input, int stream, uint32_t *pts )
+{
+  struct priv_t	*priv = input->sys.priv;
 
-  return decoded;
+  if( pts )	*pts = (uint32_t)((double)( priv->pIO->Tell( priv->pIO ) -
+				priv->data_offset ) / priv->byterate * 1000 );
+
+  return XMM_RET_OK;
 }
 
 /*
  * Seek to position in audio stream
  */
-static int wave_AudioSeek( XMM_PluginInput *input, int stream, uint32_t sample )
+static int wave_AudioSeek( XMM_PluginInput *input, int stream, uint32_t pts )
 {
   struct priv_t	*priv = input->sys.priv;
-  uint32_t	bytes;
+  double	bytes;
 
   if( priv->seekable == 0 )
-	return xmm_SetError( input->sys.xmm, XMM_ERR_NOTSUPPORTED, "(WAVE) Stream is not seekable." );
+	return xmm_SetError( input->sys.xmm, XMM_RET_NOTSUPPORTED, "(WAVE) Stream is not seekable." );
 
-  /* Calculate compressed data position */
-  bytes = ( sample / priv->ai.format.samprate ) * priv->ai.bitrate / 8;
-
-  /* Align bytes to a multiple of the blocksize */
-  bytes = alignUP( bytes, priv->blockSize );
-
-  /* Update cSample */
-  priv->cSample = bytes / ( priv->ai.bitrate / 8 ) * priv->ai.format.samprate;
+  bytes = (double)pts * priv->byterate / 1000;
 
   /* I/O Seek */
-  return priv->pIO->Seek( priv->pIO, priv->data_offset + bytes, XMM_SEEK_SET );
+  return priv->pIO->Seek( priv->pIO, priv->data_offset + (uint32_t)bytes, XMM_SEEK_SET );
 }
 
 /*
@@ -447,7 +483,7 @@ static int wave_VideoInfo( XMM_PluginInput *input, int stream, XMM_VideoInfo *vi
   return XMM_RET_NOTSUPPORTED;
 }
 
-static int wave_VideoRead( XMM_PluginInput *input, int stream, uint8_t *buffer[] )
+static int wave_VideoRead( XMM_PluginInput *input, int stream, uint8_t *buffer[], int flags )
 {
   return XMM_RET_NOTSUPPORTED;
 }
@@ -494,7 +530,6 @@ static int wave_FileInfo( void *xmm, char *filename, XMM_ClipInfo *ci )
 /*
  * Plugin data
  */
-
 XMM_PluginInput	plugin_info = {	{ NULL,
 				XMM_PLUGIN_ID,
 				XMM_PLUGIN_TYPE_INPUT,
@@ -503,12 +538,12 @@ XMM_PluginInput	plugin_info = {	{ NULL,
 				"",
 				"WAVE",
 				"Input: WAVE",
-				"Copyright (c) 2000, 2001 by Arthur Kleer",
+				"Copyright (c) 2000 Arthur Kleer",
 				NULL, NULL },
 				wave_Init, wave_Open, wave_Close,
 				wave_Control, wave_Seek, wave_Info, 
 				wave_AudioStreams, wave_AudioInfo,
-				wave_AudioRead, wave_AudioSeek,
+				wave_AudioRead, wave_AudioPTS, wave_AudioSeek,
 				wave_VideoStreams, wave_VideoInfo,
 				wave_VideoRead, wave_VideoPTS, wave_VideoSeek,
 				wave_Check, wave_FileInfo };
@@ -569,7 +604,6 @@ static int SEEK( XMM_PluginIO *pIO, long offset, int seekable )
 /*
  * Read WAVE format header
  */
-
 static int Read_fmt( XMM_PluginIO *pIO, WAVEFORMATEX *wave_fmt_head, uint32_t size, unsigned char **wave_fmt_extra )
 {
   if( size < 0x10 )
@@ -586,6 +620,9 @@ static int Read_fmt( XMM_PluginIO *pIO, WAVEFORMATEX *wave_fmt_head, uint32_t si
   wave_fmt_head->wBitsPerSample		= READ16( pIO );
   wave_fmt_head->cbSize			= 0;
 
+  if( wave_fmt_head->wBitsPerSample == 0 )
+	wave_fmt_head->wBitsPerSample = 0x10;
+
   if( wave_fmt_head->wFormatTag != 0x01 )
   {
 	if( size < 0x12 )
@@ -619,13 +656,110 @@ static int Read_fmt( XMM_PluginIO *pIO, WAVEFORMATEX *wave_fmt_head, uint32_t si
 	}
   }
 
-  xmm_logging( 2, "WAVE! fmt: FormatTag = %x\n", wave_fmt_head->wFormatTag );
-  xmm_logging( 2, "WAVE! fmt: Channels = %i\n", wave_fmt_head->nChannels );
-  xmm_logging( 2, "WAVE! fmt: Samples/Sec = %li\n",  wave_fmt_head->nSamplesPerSec );
-  xmm_logging( 2, "WAVE! fmt: Avg Bytes/Sec = %li\n", wave_fmt_head->nAvgBytesPerSec );
-  xmm_logging( 2, "WAVE! fmt: BlockAlign = %i\n", wave_fmt_head->nBlockAlign );
-  xmm_logging( 2, "WAVE! fmt: Bits / Sample = %i\n", wave_fmt_head->wBitsPerSample );
-  xmm_logging( 2, "WAVE! fmt: Extra data: %i bytes\n", wave_fmt_head->cbSize );
+#ifdef VERBOSE
+  xmm_logging( 2, "WAVE! wfx: wFormatTag = 0x%x\n", wave_fmt_head->wFormatTag );
+  xmm_logging( 2, "WAVE! wfx: nChannels = %i\n", wave_fmt_head->nChannels );
+  xmm_logging( 2, "WAVE! wfx: nSamplesPerSec = %li\n", wave_fmt_head->nSamplesPerSec );
+  xmm_logging( 2, "WAVE! wfx: nAvgBytesPerSec = %li\n", wave_fmt_head->nAvgBytesPerSec );
+  xmm_logging( 2, "WAVE! wfx: nBlockAlign = %i\n", wave_fmt_head->nBlockAlign );
+  xmm_logging( 2, "WAVE! wfx: wBitsPerSample = %i\n", wave_fmt_head->wBitsPerSample );
+  xmm_logging( 2, "WAVE! wfx: cbSize = %i\n", wave_fmt_head->cbSize );
+#endif
 
   return 1;
 }
+
+/*
+ * Read INFO chunk
+ */
+static int Read_INFO_chunk( XMM_PluginIO *pIO, XMM_ClipInfo *ci, uint32_t dwName, uint32_t dwSize )
+{
+  char	info_buffer[4096];
+
+  pIO->Read( pIO, info_buffer, dwSize, 1 );
+
+   switch( dwName )
+   {
+	case RIFF_ID_INAM:
+		strncpy( ci->name, info_buffer, XMM_CIL_NAME - 1 );
+		ci->name[XMM_CIL_NAME - 1] = '\0';
+		break;
+
+	case RIFF_ID_ISBJ:
+		strncpy( ci->content, info_buffer, XMM_CIL_CONTENT - 1 );
+		ci->content[XMM_CIL_CONTENT - 1] = '\0';
+		break;
+
+	case RIFF_ID_IART:
+		strncpy( ci->author, info_buffer, XMM_CIL_AUTHOR - 1 );
+		ci->author[XMM_CIL_AUTHOR - 1] = '\0';
+		break;
+
+	case RIFF_ID_ICOP:
+		strncpy( ci->copyright, info_buffer, XMM_CIL_COPYRIGHT - 1 );
+		ci->copyright[XMM_CIL_COPYRIGHT - 1] = '\0';
+		break;
+
+	case RIFF_ID_ISFT:
+		strncpy( ci->software, info_buffer, XMM_CIL_SOFTWARE - 1 );
+		ci->software[XMM_CIL_SOFTWARE - 1] = '\0';
+		break;
+
+	case RIFF_ID_ICMT:
+		strncpy( ci->comment, info_buffer, XMM_CIL_COMMENT - 1 );
+		ci->comment[XMM_CIL_COMMENT - 1] = '\0';
+		break;
+
+	case RIFF_ID_ICRD:
+		strncpy( ci->date + 4, info_buffer, 4 );
+		strncpy( ci->date + 2, info_buffer + 5, 2 );
+		strncpy( ci->date, info_buffer + 8, 2 );
+		ci->date[XMM_CIL_DATE - 1] = '\0';
+		break;
+
+	default:
+		xmm_logging( 2, "AVI! Ignoring INFO chunk 0x%x [%s] [%li bytes]\n", dwName, xmm_FOURCC_string(dwName), dwSize );
+		break;
+  }
+
+  if( dwSize & 1 )	READ8( pIO );
+  return 1;
+}
+
+/*
+ * Misc
+ */
+
+uint32_t codec_ms2xmm( uint16_t ms_codec )
+{
+  int		xmm_fmt_idx;
+  struct format_conv
+  {
+	int	xmm;
+	int	ms;
+  } format_table[] =
+    {
+	{ XMM_AUDIO_CODEC_PCM, WAVE_FORMAT_PCM },
+	{ XMM_AUDIO_CODEC_IEEE, WAVE_FORMAT_IEEE_FLOAT },
+	{ XMM_AUDIO_CODEC_ULAW, WAVE_FORMAT_MULAW },
+	{ XMM_AUDIO_CODEC_ALAW, WAVE_FORMAT_ALAW },
+	{ XMM_AUDIO_CODEC_MPEG, WAVE_FORMAT_MPEGLAYER3 },
+	{ XMM_AUDIO_CODEC_AC3, WAVE_FORMAT_AC3 },
+	{ XMM_AUDIO_CODEC_GSM610MS, WAVE_FORMAT_GSM610 },
+	{ XMM_AUDIO_CODEC_G721, WAVE_FORMAT_G721_ADPCM },
+	{ XMM_AUDIO_CODEC_G723, WAVE_FORMAT_G723_ADPCM },
+	{ XMM_AUDIO_CODEC_G726, WAVE_FORMAT_G726_ADPCM },
+	{ XMM_AUDIO_CODEC_WMA1, WAVE_FORMAT_WMA1 },
+	{ XMM_AUDIO_CODEC_WMA2, WAVE_FORMAT_WMA2 },
+	{ XMM_AUDIO_CODEC_ADPCM_MS, WAVE_FORMAT_ADPCM },
+	{ XMM_AUDIO_CODEC_ADPCM_IMA, WAVE_FORMAT_DVI_ADPCM },
+	{ XMM_AUDIO_CODEC_VOX_META, 0x75 },
+	{ 0, 0 }
+    };
+
+  /* Check format */
+  for( xmm_fmt_idx = 0; format_table[xmm_fmt_idx].xmm; xmm_fmt_idx++ )
+	if( ms_codec == format_table[xmm_fmt_idx].ms )	break;
+
+  return format_table[xmm_fmt_idx].xmm;
+}
